// John Gaffrey, Peter Kinnunen
// Univ. of Michigan Dearborn
// CIS 487
// HUNTER SPECTRE

// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  

// you must #define INITGUID if not done elsewhere
#define INITGUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <objbase.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <fstream.h>
#include <vector>
#include <algorithm>

#include <ddraw.h>     // directX includes
#include <dinput.h>
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include "T3DLIB1.H"
#include "T3DLIB2.H"
#include "T3DLIB3.H"

//----------------------------------------------------------------------------
// DEFINES ////////////////////////////////////////////////
//-------------------------------------------------------------------------------
// defines for windows 
#define WINDOW_CLASS_NAME "WINCLASS1"

// default screen size
#define SCREEN_WIDTH    1024  // size of screen
#define SCREEN_HEIGHT   768
#define SCREEN_BPP      8    // bits per pixel
#define BITMAP_ID            0x4D42 // universal id for a bitmap
#define MAX_COLORS_PALETTE   256

// Game defines

// number of level textures, length of map in x/y directions in tiles, total # of maps
#define NUM_TEXTURES         56
#define MAP_LENGTH_X	     40
#define MAP_LENGTH_Y	     56
#define MAP_NUMBER			 3

// Objects
#define HP_ITEM				85
#define MACHINEAMMO_ITEM	86
#define ENERGYAMMO_ITEM		87
#define FLAMEAMMO_ITEM		88
#define BARREL_ITEM			89
#define KEY_ITEM			90
#define ZOMBIE				91
#define LARGE_EXPLOSION		92
#define SMALL_EXPLOSION		93
#define BLOB				94
#define SPIDER_BLOB			95
#define BEHEMOTH			96
// weapons
#define MACHINE_GUN		1
#define ENERGY_GUN		2
#define FLAME_GUN		3

// player needs to find this number of key items in maze levels
#define KEY_ITEM_GOAL   5

// Object States/animation series
#define SOUTH        0
#define SWEST        1
#define WEST		 2
#define NWEST		 3
#define NORTH        4
#define NEAST        5
#define EAST         6
#define SEAST		 7
#define ASOUTH       8
#define ASWEST       9
#define AWEST		 10
#define ANWEST		 11
#define ANORTH       12
#define ANEAST       13
#define AEAST        14
#define ASEAST		 15
#define DEATH		 16
int anims[17][4] = { {0,1,0,2},
                    {0+3,1+3,0+3,2+3},
                    {0+6,1+6,0+6,2+6},
                    {0+9,1+9,0+9,2+9},
                    {0+12,1+12,0+12,2+12},
                    {0+15,1+15,0+15,2+15},
                    {0+18,1+18,0+18,2+18},
                    {0+21,1+21,0+21,2+21},
					{1,2,1,2},
					{1+3,2+3,1+3,2+3},
                    {1+6,2+6,1+6,2+6},
                    {1+9,2+9,1+9,2+9},
                    {1+12,2+12,1+12,2+12},
                    {1+15,2+15,1+15,2+15},
                    {1+18,2+18,1+18,2+18},
					{1+21,2+21,1+21,2+21},
                    {24,25,26,27} };

//----------------------------------------------------------------------------
// GLOBALS ////////////////////////////////////////////////
//-----------------------------------------------------------------------------
// windows vars
HWND      main_window_handle = NULL; // globally track main window
int       window_closed      = 0;    // tracks if window is closed
HINSTANCE main_instance      = NULL; // globally track hinstance

char buffer[80];                // used to print text

// holds integers representing map tiles, and vector list of game objects in map tile
typedef struct GAME_OBJECT_TYP
{
	int type; // object type, ex. enemy, environment object, bullet/blast
	int x, y; // position to draw
	int x0, y0; //upper left hitbox corner
	int x1, y1; // lower right hitbox corner
	int dx, dy; // velocity/movement change
	int wx, wy; // current location in world 2d array
	int direction; //current direction
	int curr_frame; // current animation
	int anim_max; // max number of frames
	int delay;	// used to time animation change
	int commit;	  // counter, for how long object stays on course before new direction
	int stuck;	  // counter, if two game enemies overlap, leads to one disappearing for a moment
	bool visible;	// used to check if collision with object should take place, ex. stuck enemy
	bool hit;		// if game object/enemy is hit by a weapon, used for object AI function,
	bool alive;		// checks if objects death animation should only be drawn,
	int hp;		// health points, if object should cease
	int attack;		// counter, for when enemy can attack player, or refresh attack rate
	int special1;	// misc counters for various special abilities of a game object
	int special2;  
	float newx, newy, edx, edy; // float values, used specifically for weapon objects, bullets/energy
					    // so that appropiate direction dx/dy can be saved as well as current 
					    // incremental position

} GAME_OBJECT;

// when an object enters a new map tile vector/list, list sorted based on position in map tile
// so objects above others have AI computed first, as well as drawn first, prevents objects overlapping
bool Sort_Objects(GAME_OBJECT &left, GAME_OBJECT &right) {
	return left.y1 < right.y1;
}

// world array holds this type, contains map tile type, and vector with associated game objects
typedef struct MAP_TILE_TYP
{
	int type;  
	std::vector<GAME_OBJECT> locallist; 

} MAP_TILE;


// Game Globals

int sounds[20];     // holds sound values

BOB          tilemaps,     // texture memory
			 background,	// title/story/ending backgrounds
			 player,		// main character
			 target,		// mouse cursor
			 weapons,		// weapon ammo/weapon richochet/item graphics/explosions
			 explosions,
			 infobar,
			 zombie,		// enemies
			 blob,
			 spider_blob,
			 behemoth;
int world_x = 0,	// current position of window, upper left corner
	world_y = 0;
int player_x = 0,   // current position player
    player_y = 0;
int mouse_x = 0,	  // current mouse coord. relative to screen
	mouse_y = 0;
int player_dx = 0,   // current dx/dy changes for player movements
	player_dy = 0;
bool violence = true;   // if death animations of enemies should take place
bool restart = false;	// if player chooses to restart after death
bool player_firing = false; // if player is firing weapon, for showing weapon firing animation
int player_moving = 0;  //tracks player movement
int player_health = 0;   // current player health, <=0 game over
int player_weapon = MACHINE_GUN;   // current weapon type selected by user, default select, num keys 1-3 selects
int player_weapon_delay = 0;     // counter for weapon delay between shots, each weapon creates unique delay,
					   // or refresh rate, 0 = able to fire
int machine_gun_ammo = 100;      // ammo left for certain weapons, game defaults init
int energy_gun_ammo = 10;
int flame_gun_ammo = 0;
int key_item = 0;				// number of key items needed for player to retrieve

// Holds MAP_TILE structs representing tile bitmap 64x64, total maps
MAP_TILE world[MAP_NUMBER][MAP_LENGTH_Y][MAP_LENGTH_X];
int current_level = 0;        // current map to access in world
int current_background = 0;	  // current background to display... title, story, ending

//------------------------------------------------------------------------------
// PROTOTYPES /////////////////////////////////////////////
//---------------------------------------------------------------------------
// game console
int Game_Init(void *parms=NULL, int num_parms = 0);
int Game_Shutdown(void *parms=NULL, int num_parms = 0);
int Game_Main(void *parms=NULL,  int num_parms = 0);

// game functions
// init maps, objects
void Init_Maps();
// load animation of player facing mouse target
void Pivot_Player();
// find direction from x0, y0 to x1, y1, return 0-7 corresponding to south - south east direction
int Slope_Direction(int x0, int y0, int x1, int y1);
// prevent player from moving into walls/enemies
void Player_Collision();
// detect if object, specifically an enemy collides with another
int Object_Collision_Object(int x0,int y0,int x1,int y1, int direction, int velocity);
// prevents an object, or enemy from moving  into walls
void Object_Collision_Wall(GAME_OBJECT &gameobject);
// checks if two specific rectangles overlap, usually hit box rects centered around objects legs/feet
int Rect_Collision(int x0,int y0,int x1,int y1,int x2,int y2,int x3,int y3);
// creates weapon object shot by player, if ammo for weapon exists
void Weapon_Attack();
// these 3 determine trajectory of weapon, damage, weapon ricochet
void Machine_Gun_AI(GAME_OBJECT &gameobject, int index);
void Energy_Gun_AI(GAME_OBJECT &gameobject, int index);
void Flame_Gun_AI(GAME_OBJECT &gameobject, int index);
// if player picks up an item, ammo/hp/key item, and appropiate ammo/health changes
void Item_AI(GAME_OBJECT &gameobject, int index);
// carries out an explosion object, and corresponding ai, then destroy explosion object
void Explosion_AI(GAME_OBJECT &gameobject, int index);
// checks all map section tiles lists for objects nearby to player, onscreen and near off
// processes AI for each
void Game_AI();
// these calculat AI for enemies
void Zombie_AI(GAME_OBJECT &gameobject , int index);
void Blob_AI(GAME_OBJECT &gameobject , int index);
void Spider_Blob_AI(GAME_OBJECT &gameobject , int index);
void Behemoth_AI(GAME_OBJECT &gameobject , int index);
// checks if variables to access world 3d array are not out of bounds
void Fix_Bounds(int &x, int &y);
//------------------------------------------------------------------------------
// CALLBACK //////////////////////////////////////////////
//--------------------------------------------------------------------------------
LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT		ps;		// used in WM_PAINT
HDC				hdc;	// handle to a device context

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here
        // return success
		return(0);
		} break;
   
	case WM_PAINT: 
		{
		// simply validate the window 
   	    hdc = BeginPaint(hwnd,&ps);	 
        
        // end painting
        EndPaint(hwnd,&ps);

        // return success
		return(0);
   		} break;

	case WM_DESTROY: 
		{

		// kill the application, this sends a WM_QUIT message 
		PostQuitMessage(0);

        // return success
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc
//-------------------------------------------------------------------------
// WINMAIN ////////////////////////////////////////////////
//-------------------------------------------------------------------------
int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{

	WNDCLASSEX winclass; // this will hold the class we create
	HWND	   hwnd;	 // generic window handle
	MSG		   msg;		 // generic message
	HDC        hdc;      // graphics device context

	// first fill in the window class stucture
	winclass.cbSize         = sizeof(WNDCLASSEX);
	winclass.style			= CS_DBLCLKS | CS_OWNDC | 
							  CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc	= WindowProc;
	winclass.cbClsExtra		= 0;
	winclass.cbWndExtra		= 0;
	winclass.hInstance		= hinstance;
	winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW); 
	winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL;
	winclass.lpszClassName	= WINDOW_CLASS_NAME;
	winclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);

	// save hinstance in global
	main_instance = hinstance;

	// register the window class
	if (!RegisterClassEx(&winclass))
		return(0);

	// create the window
	if (!(hwnd = CreateWindowEx(NULL,                  // extended style
								WINDOW_CLASS_NAME,     // class
								"Game", // title
								WS_POPUP | WS_VISIBLE,
					 			0,0,	  // initial x,y
								SCREEN_WIDTH, SCREEN_HEIGHT,  // initial width, height
								NULL,	  // handle to parent 
								NULL,	  // handle to menu
								hinstance,// instance of this application
								NULL)))	// extra creation parms
	return(0);

	// save main window handle
	main_window_handle = hwnd;

	// initialize game here
	Game_Init();

	// enter main event loop
	while(TRUE)
		{
		// test if there is a message in queue, if so get it
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		   { 
		   // test if this is a quit
		   if (msg.message == WM_QUIT)
			   break;
		
		   // translate any accelerator keys
		   TranslateMessage(&msg);

		   // send the message to the window proc
		   DispatchMessage(&msg);
		   } // end if
    
		   // main game processing goes here
		   Game_Main();
       
		} // end while
	DSound_Stop_All_Sounds();
	// closedown game here
	Game_Shutdown();

	// return to Windows like this
	return(msg.wParam);

} // end WinMain

// GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

int Game_Init(void *parms,  int num_parms)
{
	// for-loop counter
	int index;
	// init mouse position
	mouse_x = (SCREEN_HEIGHT >> 1);
	mouse_y = (SCREEN_HEIGHT >> 1);
	// initialize maps, tiles, objects
	Init_Maps();
	
	// initialize directdraw
	DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

	//-------------map tiles----------------------------------------------------------
	Load_Bitmap_File(&bitmap8bit, "bitmaps/tilemaps.bmp");

	// set the palette to background image palette
	Set_Palette(bitmap8bit.palette);

	// create the texture tile map bob
	if (!Create_BOB(&tilemaps,0,0,64,64,NUM_TEXTURES, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME))
	   return(0);

	// load each texture tile map bitmap into the texture BOB object
	for (index = 0; index < NUM_TEXTURES; index++)
		Load_Frame_BOB(&tilemaps,&bitmap8bit,index,index%14,index/14,BITMAP_EXTRACT_MODE_CELL); 

	// unload the texture map bitmap
	Unload_Bitmap_File(&bitmap8bit);
	//-------------info bar----------------------------------------------------------
	Load_Bitmap_File(&bitmap8bit, "bitmaps/infobar.bmp");

	// set the palette to background image palette
	Set_Palette(bitmap8bit.palette);

	// create the texture tile map bob
	if (!Create_BOB(&infobar,0,0,1024,64,NUM_TEXTURES, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME))
	   return(0);

	// load each texture tile map bitmap into the texture BOB object
	Load_Frame_BOB(&infobar,&bitmap8bit,0,0,0,BITMAP_EXTRACT_MODE_ABS); 

	// unload the texture map bitmap
	Unload_Bitmap_File(&bitmap8bit);
	//-------------Title, Story and Ending backgrounds--------------------------------
	if (!Create_BOB(&background,0,0,800,600,3, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME))
	   return(0);
	// title
	Load_Bitmap_File(&bitmap8bit, "bitmaps/title.bmp");
	Load_Frame_BOB(&background,&bitmap8bit,0,0,0,BITMAP_EXTRACT_MODE_ABS); 
	Unload_Bitmap_File(&bitmap8bit);
	
	// story 
	Load_Bitmap_File(&bitmap8bit, "bitmaps/story.bmp");
	Load_Frame_BOB(&background,&bitmap8bit,1,0,0,BITMAP_EXTRACT_MODE_ABS); 
	Unload_Bitmap_File(&bitmap8bit);
	
	// ending
	Load_Bitmap_File(&bitmap8bit, "bitmaps/end.bmp");
	Load_Frame_BOB(&background,&bitmap8bit,2,0,0,BITMAP_EXTRACT_MODE_ABS); 
	Unload_Bitmap_File(&bitmap8bit);

	//-------------main character--------------------------------------------------------
	Load_Bitmap_File(&bitmap8bit, "bitmaps/maincharacter.bmp");
	
	// create the main character bob
	if (!Create_BOB(&player,0,0,122,100,25, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM))
	   return(0);

	// load each texture tile map bitmap into the texture BOB object
	for (index = 0; index < 25; index++)
		Load_Frame_BOB(&player,&bitmap8bit,index,index%8,index/8,BITMAP_EXTRACT_MODE_CELL); 

	Set_Anim_Speed_BOB(&player, 6);
	Load_Animation_BOB(&player,0,4,anims[0]);
	Load_Animation_BOB(&player,1,4,anims[1]);
	Load_Animation_BOB(&player,2,4,anims[2]);
	Load_Animation_BOB(&player,3,4,anims[3]);
	Load_Animation_BOB(&player,4,4,anims[4]);
	Load_Animation_BOB(&player,5,4,anims[5]);
	Load_Animation_BOB(&player,6,4,anims[6]);
	Load_Animation_BOB(&player,7,4,anims[7]);
	// unload the texture map bitmap
	Unload_Bitmap_File(&bitmap8bit);
	//----------------target cursor----------------------------------------------------
	Load_Bitmap_File(&bitmap8bit, "bitmaps/cursor.bmp");
	if (!Create_BOB(&target,mouse_x,mouse_y,64,68,1, 
					BOB_ATTR_VISIBLE | BOB_ATTR_SINGLE_FRAME))
		return 0;

	Load_Frame_BOB(&target,&bitmap8bit,0,0,0,BITMAP_EXTRACT_MODE_CELL);
	Unload_Bitmap_File(&bitmap8bit);

	//----------------weapon types-------------------------------------------------------
	// weapon bitmap effects, ammo/health objects
	Load_Bitmap_File(&bitmap8bit, "bitmaps/weapons.bmp");
	if (!Create_BOB(&weapons,mouse_x,mouse_y,90,90,36, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM))
		return 0;
	Set_Anim_Speed_BOB(&weapons, 6);
	for(index = 0; index < 36; index++)
		Load_Frame_BOB(&weapons,&bitmap8bit,index,index%12,index/12,BITMAP_EXTRACT_MODE_CELL);
	Unload_Bitmap_File(&bitmap8bit);
	//----------------explosion types-------------------------------------------------------
	// weapon bitmap effects, ammo/health objects
	Load_Bitmap_File(&bitmap8bit, "bitmaps/explosions.bmp");
	if (!Create_BOB(&explosions,0,0,360,360,17, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM))
		return 0;
	Set_Anim_Speed_BOB(&weapons, 6);
	for(index = 0; index < 17; index++)
		Load_Frame_BOB(&explosions,&bitmap8bit,index,index%5,index/5,BITMAP_EXTRACT_MODE_CELL);
	Unload_Bitmap_File(&bitmap8bit);
	//----------------zombie------------------------------------------------------------//
	Load_Bitmap_File(&bitmap8bit, "bitmaps/zombie.bmp");
	
	if (!Create_BOB(&zombie,0,0,100,100,28, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM))
	   return(0);

	for (index = 0; index < 28; index++)
		Load_Frame_BOB(&zombie,&bitmap8bit,index,index%8,index/8,BITMAP_EXTRACT_MODE_CELL); 

	Set_Anim_Speed_BOB(&zombie, 6);

	Unload_Bitmap_File(&bitmap8bit);
	//----------------blob------------------------------------------------------------//
	Load_Bitmap_File(&bitmap8bit, "bitmaps/blob.bmp");
	
	if (!Create_BOB(&blob,0,0,39,45,17, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM))
	   return(0);

	for (index = 0; index < 17; index++)
		Load_Frame_BOB(&blob,&bitmap8bit,index,index%17,index/17,BITMAP_EXTRACT_MODE_CELL); 

	Set_Anim_Speed_BOB(&blob, 3);

	Unload_Bitmap_File(&bitmap8bit);
	//----------------spider_blob-----------------------------------------------------------//
	Load_Bitmap_File(&bitmap8bit, "bitmaps/spider_blob.bmp");
	
	if (!Create_BOB(&spider_blob,0,0,160,160,28, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM))
	   return(0);

	for (index = 0; index < 28; index++)
		Load_Frame_BOB(&spider_blob,&bitmap8bit,index,index%6,index/6,BITMAP_EXTRACT_MODE_CELL); 

	Set_Anim_Speed_BOB(&spider_blob, 6);

	Unload_Bitmap_File(&bitmap8bit);
	//----------------behemoth------------------------------------------------------------//
	Load_Bitmap_File(&bitmap8bit, "bitmaps/behemoth.bmp");
	
	if (!Create_BOB(&behemoth,0,0,230,230,28, 
					BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM))
	   return(0);

	for (index = 0; index < 28; index++)
		Load_Frame_BOB(&behemoth,&bitmap8bit,index,index%4,index/4,BITMAP_EXTRACT_MODE_CELL); 

	Set_Anim_Speed_BOB(&behemoth, 6);

	Unload_Bitmap_File(&bitmap8bit);
	//------------------------------------------------------------------------------------
	// initialize directinput
	DInput_Init();

	// acquire the keyboard
	DInput_Init_Keyboard();

	// acquire the mouse
	DInput_Init_Mouse();

	// init sound
	DSound_Init();

	// init sounds, and frequency/volume if needed
	sounds[0] = DSound_Load_WAV("sounds/bkgsound.wav");
		DSound_Set_Volume(sounds[0],90);
		DSound_Play(sounds[0], DSBPLAY_LOOPING);
	sounds[1] = DSound_Load_WAV("sounds/flamethrower.wav");
		DSound_Set_Volume(sounds[1],80);
		DSound_Set_Freq(sounds[1],20000);
	sounds[2] = DSound_Load_WAV("sounds/barrel_exp.wav");
	sounds[3] = DSound_Load_WAV("sounds/wall_exp.wav");
		DSound_Set_Volume(sounds[3],70);
	sounds[4] = DSound_Load_WAV("sounds/energy.wav");
		DSound_Set_Volume(sounds[4],60);
	sounds[5] = DSound_Load_WAV("sounds/player_hit.wav");
	sounds[6] = DSound_Load_WAV("sounds/heartbeat.wav");
		DSound_Set_Volume(sounds[6],75);
		DSound_Set_Freq(sounds[6],15000);
	sounds[7] = DSound_Load_WAV("sounds/energy_blast.wav");
		DSound_Set_Volume(sounds[7],60);
	sounds[8] = DSound_Load_WAV("sounds/machinegun.wav");
		DSound_Set_Volume(sounds[8],30);
	sounds[9] = DSound_Load_WAV("sounds/bulletricochet.wav");
		DSound_Set_Volume(sounds[9],30);
		DSound_Set_Freq(sounds[9],6000);
	sounds[10] = DSound_Load_WAV("sounds/zombie.wav");
		DSound_Set_Freq(sounds[10],6000);
		DSound_Set_Volume(sounds[10],60);
	sounds[11] = DSound_Load_WAV("sounds/zombie_death.wav");
		DSound_Set_Freq(sounds[11],3000);
	sounds[12] = DSound_Load_WAV("sounds/behemoth_death.wav");
		DSound_Set_Freq(sounds[12],6000);
		DSound_Set_Volume(sounds[12],80);
	sounds[13] = DSound_Load_WAV("sounds/behemoth_walking.wav");
		DSound_Set_Freq(sounds[13],6000);
		DSound_Set_Volume(sounds[13],80);
	sounds[14] = DSound_Load_WAV("sounds/reload.wav");
	sounds[15] = DSound_Load_WAV("sounds/blob.wav");
		DSound_Set_Freq(sounds[15],7000);
		DSound_Set_Volume(sounds[15],80);
	sounds[16] = DSound_Load_WAV("sounds/blob_hit.wav");
		DSound_Set_Freq(sounds[16],7000);
		DSound_Set_Volume(sounds[16],80);
	sounds[17] = DSound_Load_WAV("sounds/spider_blob_death.wav");
		DSound_Set_Freq(sounds[17],7000);
		DSound_Set_Volume(sounds[17],80);
	sounds[18] = DSound_Load_WAV("sounds/spider_blob_create.wav");
		DSound_Set_Freq(sounds[18],10000);
	sounds[19] = DSound_Load_WAV("sounds/spider_blob_attack.wav");
		DSound_Set_Volume(sounds[19],80);
		DSound_Set_Freq(sounds[19],15000);
	sounds[20] = DSound_Load_WAV("sounds/spider_blob.wav");
		DSound_Set_Volume(sounds[20],80);

	// set clipping rectangle to screen extents so mouse cursor
	// doens't mess up at edges
	RECT screen_rect = {0,0,screen_width,screen_height}; // 32 pixels at the bottom for controls

	// notice at bottom of screen a blank rect, this would be for your
	// control panel for example
	lpddclipper = DDraw_Attach_Clipper(lpddsback,1,&screen_rect);

	// hide the mouse
	ShowCursor(FALSE);

	// return success
	return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms,  int num_parms)
{
	// this function is where you shutdown your game and
	// release all resources that you allocated
	for(int z = 0; z < MAP_NUMBER; z++) {
		for(int y = 0; y < MAP_LENGTH_Y; y++) {
			for(int x = 0; x < MAP_LENGTH_X; x++) {
				world[z][y][x].locallist.clear();
			}
		}
	}
	
	// kill texture memory, release input/sound devices
	Destroy_BOB(&tilemaps);
	Destroy_BOB(&background);
	Destroy_BOB(&player);
	Destroy_BOB(&target);
	Destroy_BOB(&weapons);
	Destroy_BOB(&zombie);
	Destroy_BOB(&blob);
	Destroy_BOB(&spider_blob);
	Destroy_BOB(&behemoth);
	Destroy_BOB(&explosions);
	Destroy_BOB(&infobar);

	DInput_Release_Keyboard();
	DInput_Release_Mouse();

	DSound_Delete_All_Sounds();
	DSound_Shutdown();

	// shutdown directdraw
	DDraw_Shutdown();

	// return success
	return(1);
} // end Game_Shutdown

///////////////////////////////////////////////////////////

int Game_Main(void *parms, int num_parms)
{
	// this is the workhorse of your game it will be called
	// continuously in real-time this is like main() in C
	// all the calls for you game go here!
	
	player_moving = 0;	// reset
	int index, index_x, index_y;  // looping vars
	int start_map_x,start_map_y,  // map positions
		end_map_x,end_map_y; 

	int offset_x, offset_y;       // pixel offsets within cell

	// read keyboard
	DInput_Read_Keyboard();

	// read mouse
	DInput_Read_Mouse();

	// check of user is trying to exit
	if (KEY_DOWN(VK_ESCAPE) || keyboard_state[DIK_ESCAPE])
		PostMessage(main_window_handle, WM_DESTROY,0,0);

	// start the timing clock
	Start_Clock();

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);
	//---------------------------------------------------------------------------------------------
	////////////////////////////title and story/////////////////////////////////////////////
	//-------------------------------------------------------------------------------------------
	// background 0 title, 1 initial story screen
	if(current_background == 0 || current_background == 1) {

		background.curr_frame = current_background;
		Draw_Scaled_BOB(&background, SCREEN_WIDTH, SCREEN_HEIGHT, lpddsback);
		// user passes current title,story screen
		if(keyboard_state[DIK_RETURN]) {
			current_background += 1;
		}
		// display story
		if(current_background == 1) {
			background.curr_frame = current_background;
			player_health = 100;
		}
	}
	//---------------------------------------------------------------------------------------------
	//////////////////////////// end of story/////////////////////////////////////////////
	//-------------------------------------------------------------------------------------------
	// users has entered end of game, goes back to title screen when user hit enter/return
	else if(key_item == KEY_ITEM_GOAL) {
		background.curr_frame = 2;
			Draw_Scaled_BOB(&background, SCREEN_WIDTH, SCREEN_HEIGHT, lpddsback);
			if(keyboard_state[DIK_RETURN]) {
					Init_Maps();
					current_background = 0;
					player_health = 100;
					key_item = 0;
			}
	}
	//---------------------------------------------------------------------------------------------
	////////////////////////////death/////////////////////////////////////////////
	//-------------------------------------------------------------------------------------------
	// player dies, reinitialized title screen, maps/objects
	else if(restart == true) {
		Init_Maps();
		current_background = 0;
		player_health = 100;
		restart = false;
	}
	//---------------------------------------------------------------------------------------------
	////////////////////////////if in main game/////////////////////////////////////////////
	//-------------------------------------------------------------------------------------------
	else {
	// if players health low
	if(player_health <= 25 && (DSound_Status_Sound(sounds[6]) != DSBSTATUS_PLAYING))
		DSound_Play(sounds[6]);
	// turns of death animations of objects
	if (keyboard_state[DIK_V]) {
		violence = !violence;
	}
	// check for movement (scrolling)
	// faster movement if player moving in direction of target/cursor, or two directions adjacent
	// ex. moving east (D key) updates x coordinate +3 if cursor/target is northeast, east, or southeast
	//-----------------------------------------------------------------------------------------------------------
	// check player movement, wall collision, update world and player coordinates, player velocity, weapon select/firing
	//----------------------------------------------------------------------------------------------------------
	// find mouse position
	mouse_x += (mouse_state.lX);
	mouse_y += (mouse_state.lY);
	// check if mouse out of screen
	if(mouse_x < 0)
		mouse_x = 0;
	else if(mouse_x > (SCREEN_WIDTH - 64))
		mouse_x = (SCREEN_WIDTH - 64);
	if(mouse_y < 0)
		mouse_y = 0;
	else if(mouse_y > (SCREEN_HEIGHT - 98))
		mouse_y = (SCREEN_HEIGHT - 98);
	// selects weapon
	if(keyboard_state[DIK_1])
		player_weapon = MACHINE_GUN;
	if(keyboard_state[DIK_2])
		player_weapon = ENERGY_GUN;
	if(keyboard_state[DIK_3])
		player_weapon = FLAME_GUN;

	// if player dies, allow user to exit to title screen
	if(player_health <= 0) {
		if(keyboard_state[DIK_RETURN])
			restart = true;
	}
	// prevents movement if player dies
	if(player_health > 0) {
		// reinitializes player movement
		// determines appropiate speed based on orientation, moving backwards/sideways is slower
		player_dx = player_dy = 0;
		if (keyboard_state[DIK_D]) {
			if((player.curr_animation == NEAST && !keyboard_state[DIK_S])||(player.curr_animation == EAST)
				||(player.curr_animation == SEAST && !keyboard_state[DIK_W]))
				player_dx += 3;
			player_dx += 2;
			player_moving = 1;}
		else
		if (keyboard_state[DIK_A]){	
			if((player.curr_animation == SWEST && !keyboard_state[DIK_W])||(player.curr_animation == WEST)
				||(player.curr_animation == NWEST && !keyboard_state[DIK_S]) )
				player_dx += -3;
			player_dx += -2;
			player_moving = 1;}

		if (keyboard_state[DIK_W]){	
			if((player.curr_animation == NWEST && !keyboard_state[DIK_D])||(player.curr_animation == NORTH)
				||(player.curr_animation == NEAST && !keyboard_state[DIK_A]))
				player_dy += -3;
			player_dy += -2;
			player_moving = 1;}
		else
		if (keyboard_state[DIK_S]){	
			if((player.curr_animation == SEAST && !keyboard_state[DIK_A])||(player.curr_animation == SOUTH)
				||(player.curr_animation == SWEST && !keyboard_state[DIK_D]) )
				player_dy += 3;
			player_dy += 2;
			player_moving = 1;}
		// update world coord.
		world_x += player_dx;
		world_y += player_dy;
	}
	// allows player to shoot, if alive
	if(player_health > 0) {
		// if player used mouse to shoot
		if(mouse_state.rgbButtons[0]) {
			// initialized appropiate gun ammo object
			Weapon_Attack();
			// checks delay of weapon, prevents new ammo object being created every cycle
			if(player_weapon_delay > 0)
				player_weapon_delay--;
		}
	}
	// checks player collision with walls, enemies, adjusts accordingly
	Player_Collision();
	// find player position, around feet
	player_x = world_x + (SCREEN_WIDTH>>1);
	player_y = world_y + (SCREEN_HEIGHT>>1)+20;
	//----------------------------------------------------------------------
	//////checks for world array out of bounds////
	//----------------------------------------------------------------------
	// check if player is out of bounds
/*	if(world_x < 0)
		world_x = 0;
	else if(world_x > (MAP_LENGTH_X << 6))
		world_x = (MAP_LENGTH_X << 6);
	if(world_y < 0)
		world_y = 0;
	else if(world_y > (MAP_LENGTH_Y << 6))
		world_y = (MAP_LENGTH_Y << 6);*/
	// compute starting map indices by dividing position by size of cell
	start_map_x = (world_x >> 6); // use >> 6 for speed, but this is clearer
	start_map_y = (world_y >> 6); 
	// if start of map is out of bounds of world matrix
	if(start_map_x < 0)
		start_map_x = 0;
	else if(start_map_x > MAP_LENGTH_X - 1)
		start_map_x = MAP_LENGTH_X - 1;
	if(start_map_y < 0)
		start_map_y = 0;
	else if(start_map_y > MAP_LENGTH_Y - 1)
		start_map_y = MAP_LENGTH_Y - 1;
	// compute end of map rectangle for best cast i.e. aligned on 64x64 boundary
	end_map_x = start_map_x + (SCREEN_WIDTH  >> 6) + 1;
	end_map_y = start_map_y + (SCREEN_HEIGHT >> 6) + 1;
	// if end of map is out of bounds of world matrix
	if(end_map_x < 0)
		end_map_x = 0;
	else if(end_map_x > MAP_LENGTH_X - 1)
		end_map_x = MAP_LENGTH_X - 1;
	if(end_map_y < 0)
		end_map_y = 0;
	else if(end_map_y > MAP_LENGTH_Y - 1)
		end_map_y = MAP_LENGTH_Y - 1;
	//------------------------------------------------------------------------------
	/////////blit map tiles////////////////////////////////////////////////
	//-----------------------------------------------------------------------------
	// now compute number of pixels in x,y we are within the tile, i.e
	// how much is scrolled off the edge?
	offset_x = -(world_x % 64);
	offset_y = -(world_y % 64);
	if(world_x < 0)
		offset_x = -(world_x);
	if(world_y < 0)
		offset_y = -(world_y);
	//adjust end_map_x,y for offsets
	if (offset_x)
	   end_map_x++;

	if (offset_y)
	   end_map_y++;


	// set starting position of first upper lh texture
	int texture_x = offset_x;
	int texture_y = offset_y;
	
	// if player has entered a tile for moving up or down a game level
	if(world[current_level][player_y >> 6][player_x >> 6].type == 29)
		current_level++;
	else if(world[current_level][player_y >> 6][player_x >> 6].type == 30)
		current_level--;

	// draw the current window
	for (index_y = start_map_y; index_y < end_map_y; index_y++)
		{
		for (index_x = start_map_x; index_x < end_map_x; index_x++)
			{
			// set position to blit
			tilemaps.x = texture_x;
			tilemaps.y = texture_y;
        
			// set frame
			tilemaps.curr_frame = world[current_level][index_y][index_x].type;

			// draw the texture
			Draw_BOB(&tilemaps,lpddsback);

			// update texture position
			texture_x+=64;

			} // end for map_x

		// reset x postion, update y
		texture_x =  offset_x;
		texture_y += 64;

	} // end for map_y
	
	// handle AI, and draw all game objects
	Game_AI();
	
	//----------------------------------------------------------
	//////////////// handle player/////////////////////////////
	//----------------------------------------------------------
	// pivots player towards target, sets positions of player, animates and blits
	if(player_health > 0)
		Pivot_Player();
	Set_Pos_BOB(&player, (SCREEN_WIDTH>>1)-61, (SCREEN_HEIGHT>>1)-50);
	if(player_moving == 1)
		Animate_BOB(&player);
	if(player_health <= 0)
		player.curr_frame = 24;
	Draw_BOB(&player, lpddsback);
	// display infobar
	Set_Pos_BOB(&infobar, 0, SCREEN_HEIGHT-64);
	Draw_BOB(&infobar, lpddsback);
	// display player health
	sprintf(buffer,"PLAYER HEALTH:    [%d]        ", player_health);
	if(player_health > 25)
		Draw_Text_GDI(buffer,75,SCREEN_HEIGHT - 40,RGB(255,255,255),lpddsback);
	else
		Draw_Text_GDI(buffer,75,SCREEN_HEIGHT - 40,RGB(255,0,0),lpddsback);
	// display ammo to corresponding weapon chose
	if(player_weapon == MACHINE_GUN)
		sprintf(buffer,"WEAPON:     MACHINE GUN      [%d]", machine_gun_ammo);
	else if(player_weapon == ENERGY_GUN)
		sprintf(buffer,"WEAPON:     ENERGY GUN       [%d]", energy_gun_ammo);
	else if(player_weapon == FLAME_GUN)
		sprintf(buffer,"WEAPON:     FLAME THROWER    [%d]", flame_gun_ammo);
	Draw_Text_GDI(buffer,378,SCREEN_HEIGHT - 40,RGB(255,255,255),lpddsback);
	// display number of bounty items found so far by player
	sprintf(buffer,"BOUNTY ITEMS FOUND:    [%d] of [%d]", key_item, KEY_ITEM_GOAL);
	Draw_Text_GDI(buffer,718,SCREEN_HEIGHT - 40,RGB(255,255,255),lpddsback);

	// display game over messages
	if(player_health <= 0) {
		sprintf(buffer,"GAME OVER---GAME OVER---GAME OVER---GAME OVER---GAME OVER");
		Draw_Text_GDI(buffer,350,SCREEN_HEIGHT >> 1,RGB(255,0,0),lpddsback);
		sprintf(buffer,"PRESS ENTER---PRESS ENTER---PRESS ENTER---PRESS ENTER---PRESS ENTER");
		Draw_Text_GDI(buffer,320,(SCREEN_HEIGHT >> 1) + 50,RGB(255,0,0),lpddsback);
	}

	//draw target/cursor
	// set cursor/target position
	Set_Pos_BOB(&target, mouse_x, mouse_y);
	Draw_BOB(&target,lpddsback);
	}// end of main game if stmt

	// flip the surfaces
	DDraw_Flip();

	

	// sync to 30 fps
	Wait_Clock(30);
	// return success
	return(1);

} // end Game_Main

//-------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------------------

void Init_Maps() {
	int x, y, z, objnum, temp1, temp2;         // loop indexes, objtype used for game objects in object map
	GAME_OBJECT gameobject;
	ifstream getmapdata, getobjectdata;			// open map/objects data files
	getmapdata.open("maps/levels.txt");
	getobjectdata.open("maps/objects.txt");
	// clear objects lists for all map tiles in game world
	for(z = 0; z < MAP_NUMBER; z++) {
		for(y = 0; y < MAP_LENGTH_Y; y++) {
			for(x = 0; x < MAP_LENGTH_X; x++) {
				world[z][y][x].locallist.clear();
			}
		}
	}
	machine_gun_ammo = 100;
	energy_gun_ammo = 10;
	flame_gun_ammo = 0;
	current_level = 0;
	// fills world 3d array based on number of levels, z and x/y lengths of level maps
	for(z = 0; z < MAP_NUMBER; z++) {
		for(y = 0; y < MAP_LENGTH_Y; y++) {
			for(x = 0; x < MAP_LENGTH_X; x++) {
				// stores type of tile
				getmapdata >> world[z][y][x].type;
				
				getobjectdata >> objnum;
				// clear each vector representing each map tile.. in case game restarts due to win/loss
				
				switch(objnum) {
					// initials position of player in map, sets world coordinates
					case 99: {
						world_x = (x << 6) -(SCREEN_WIDTH >> 1);
						world_y = (y << 6) -(SCREEN_HEIGHT >> 1);
						break;
							  }
					// zombie type, initialized zombie object data, puts it in initial map tile object list
					case 91: {
						gameobject.type = ZOMBIE;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.direction = rand() % 8;
						// hit box of object centered around legs/feet
						gameobject.x0 = gameobject.x + 25;
						gameobject.y0 = gameobject.y + 70;
						gameobject.x1 = gameobject.x0 + 50;
						gameobject.y1 = gameobject.y0 + 30;
						gameobject.wx = temp1 = (gameobject.x0 >> 6);
						gameobject.wy = temp2 = (gameobject.y1 >> 6);
						gameobject.anim_max = 4;
						gameobject.curr_frame = 0;
						gameobject.dx = 2;
						gameobject.dy = 2;
						gameobject.delay = 0;
						gameobject.commit = 30;
						gameobject.stuck = 0;
						gameobject.visible = true;
						gameobject.hit = false;
						gameobject.alive = true;
						gameobject.hp = 100;
						gameobject.attack = 0;
						world[z][temp2][temp1].locallist.push_back(gameobject);
						break;
							  }
					// blob type, initialized blob object data, puts it in initial map tile object list
					case 94: {
						gameobject.type = BLOB;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.wx = x;
						gameobject.wy = y;
						gameobject.direction = rand() % 8;
						// hit box of object centered around legs/feet
						gameobject.x0 = gameobject.x;
						gameobject.y0 = gameobject.y;
						gameobject.x1 = gameobject.x + 39;
						gameobject.y1 = gameobject.y + 45;
						gameobject.anim_max = 4;
						gameobject.curr_frame = 0;
						gameobject.dx = 2;
						gameobject.dy = 2;
						gameobject.delay = 0;
						gameobject.commit = 30;
						gameobject.stuck = 0;
						gameobject.visible = true;
						gameobject.hit = false;
						gameobject.alive = true;
						gameobject.hp = 100;
						gameobject.attack = 0;
						gameobject.special1 = 0;
						world[z][y][x].locallist.push_back(gameobject);
						break;
							  }
					// spider blob enemy
					case 95: {
						gameobject.type = SPIDER_BLOB;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.direction = rand() % 8;
						// hit box of object centered around legs/feet
						gameobject.x0 = gameobject.x + 30;
						gameobject.y0 = gameobject.y + 30;
						gameobject.x1 = gameobject.x0 + 100;
						gameobject.y1 = gameobject.y0 + 120;
						gameobject.wx = temp1 = (gameobject.x0 >> 6);
						gameobject.wy = temp2 = (gameobject.y1 >> 6);
						gameobject.anim_max = 4;
						gameobject.curr_frame = 0;
						gameobject.dx = 2;
						gameobject.dy = 2;
						gameobject.delay = 0;
						gameobject.commit = 0;
						gameobject.stuck = 0;
						gameobject.visible = true;
						gameobject.hit = false;
						gameobject.alive = true;
						gameobject.hp = 100;
						gameobject.attack = 0;
						gameobject.special1 = 150;
						gameobject.special2 = 10;
						world[z][temp2][temp1].locallist.push_back(gameobject);
						break;
							  }
					// behemoth type enemy
					case 96: {
						gameobject.type = BEHEMOTH;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.direction = rand() % 8;
						// hit box of object centered around legs/feet
						gameobject.x0 = gameobject.x + 50;
						gameobject.y0 = gameobject.y + 130;
						gameobject.x1 = gameobject.x0 + 130;
						gameobject.y1 = gameobject.y0 + 90;
						gameobject.wx = temp1 = (gameobject.x0 >> 6);
						gameobject.wy = temp2 = (gameobject.y1 >> 6);
						gameobject.anim_max = 4;
						gameobject.curr_frame = 0;
						gameobject.dx = 2;
						gameobject.dy = 2;
						gameobject.delay = 0;
						gameobject.commit = 0;
						gameobject.stuck = 0;
						gameobject.visible = true;
						gameobject.hit = false;
						gameobject.alive = true;
						gameobject.hp = 100;
						gameobject.attack = 0;
						gameobject.special1 = 200;
						gameobject.special2 = 0;
						world[z][temp2][temp1].locallist.push_back(gameobject);
						break;
							  }
					// create item, HP icon
					case 85: {
						gameobject.type = HP_ITEM;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.wx = x;
						gameobject.wy = y;
						gameobject.x0 = gameobject.x + 20;
						gameobject.y0 = gameobject.y + 20;
						gameobject.x1 = gameobject.x0 + 50;
						gameobject.y1 = gameobject.y0 + 50;
						gameobject.curr_frame = 12;
						gameobject.visible = false;
						gameobject.hp = 0;
						world[z][y][x].locallist.push_back(gameobject);
						break;
							}
					// create item, Machine gun ammo type
					case 86: {
						gameobject.type = MACHINEAMMO_ITEM;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.wx = x;
						gameobject.wy = y;
						gameobject.x0 = gameobject.x + 20;
						gameobject.y0 = gameobject.y + 20;
						gameobject.x1 = gameobject.x0 + 50;
						gameobject.y1 = gameobject.y0 + 50;
						gameobject.curr_frame = 13;
						gameobject.visible = false;
						gameobject.hp = 0;
						world[z][y][x].locallist.push_back(gameobject);
						break;
							}
					// create items, energy gun ammo
					case 87: {
						gameobject.type = ENERGYAMMO_ITEM;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.wx = x;
						gameobject.wy = y;
						gameobject.x0 = gameobject.x + 20;
						gameobject.y0 = gameobject.y + 20;
						gameobject.x1 = gameobject.x0 + 50;
						gameobject.y1 = gameobject.y0 + 50;
						gameobject.curr_frame = 14;
						gameobject.visible = false;
						gameobject.hp = 0;
						world[z][y][x].locallist.push_back(gameobject);
						break;
							}
					// create items, flame gun ammo
					case 88: {
						gameobject.type = FLAMEAMMO_ITEM;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.wx = x;
						gameobject.wy = y;
						gameobject.x0 = gameobject.x + 20;
						gameobject.y0 = gameobject.y + 20;
						gameobject.x1 = gameobject.x0 + 50;
						gameobject.y1 = gameobject.y0 + 50;
						gameobject.curr_frame = 15;
						gameobject.visible = false;
						gameobject.hp = 0;
						world[z][y][x].locallist.push_back(gameobject);
						break;
							}
					// create item, explosive barrel
					case 89: {
						gameobject.type = BARREL_ITEM;
						gameobject.x = (x << 6) - 20 + (rand() % 40);
						gameobject.y = (y << 6) - 20 + (rand() % 40);
						gameobject.wx = x;
						gameobject.wy = y;
						gameobject.x0 = gameobject.x + 20;
						gameobject.y0 = gameobject.y + 50;
						gameobject.x1 = gameobject.x0 + 50;
						gameobject.y1 = gameobject.y0 + 30;
						gameobject.curr_frame = 18;
						gameobject.visible = true;
						gameobject.hp = 10;
						gameobject.special1 = 20;
						world[z][y][x].locallist.push_back(gameobject);
						break;
							}
					// create key item, player finds to win game
					case 90: {
						gameobject.type = KEY_ITEM;
						gameobject.x = (x << 6);
						gameobject.y = (y << 6);
						gameobject.wx = x;
						gameobject.wy = y;
						gameobject.x0 = gameobject.x + 20;
						gameobject.y0 = gameobject.y + 20;
						gameobject.x1 = gameobject.x0 + 50;
						gameobject.y1 = gameobject.y0 + 50;
						gameobject.curr_frame = 16;
						gameobject.visible = false;
						gameobject.hp = 0;
						world[z][y][x].locallist.push_back(gameobject);
						break;
							}

					default: break;

				}
			}
		}
	}
	getmapdata.close();
	getobjectdata.close();
}

// pivots player towards direction of target/cursor, 8 directions
void Pivot_Player() {
	int direction;
	// finds direction
	// using center of screen (player) and center of target/cursor
	direction = Slope_Direction((SCREEN_WIDTH >> 1),(SCREEN_HEIGHT >> 1), mouse_x + 32, mouse_y + 34);
	// updates animation if need be
	if (player.curr_animation != direction) {
		// sets new animation, loads appropiate fram and blits player
		Set_Animation_BOB(&player, direction);
		player.curr_frame= 3 * direction;
		Draw_BOB(&player, lpddsback);
	}
	// displays weapon firing animation for current direction of player, based on direction facing
	// position chosen is just an estimate of where end of gun would be based on screen coord.
	if(player_firing == true) {
		switch(direction) {
			case SOUTH: {
				Set_Pos_BOB(&weapons, (player_x-54) - world_x, (player_y-12) - world_y);
				weapons.curr_frame = 19;
				Draw_BOB(&weapons, lpddsback);
				break;
						}
			case SWEST: {
				Set_Pos_BOB(&weapons, (player_x-100) - world_x, (player_y-45) - world_y);
				weapons.curr_frame = 19;
				Draw_BOB(&weapons, lpddsback);
				break;
						}
			case WEST: {
				Set_Pos_BOB(&weapons, (player_x-105) - world_x, (player_y-72) - world_y);
				weapons.curr_frame = 19;
				Draw_BOB(&weapons, lpddsback);
				break;
						}
			case NWEST: {
				Set_Pos_BOB(&weapons, (player_x-70) - world_x, (player_y-110) - world_y);
				weapons.curr_frame = 19;
				Draw_BOB(&weapons, lpddsback);
				break;
						}
			case NORTH: {
				Set_Pos_BOB(&weapons, (player_x-30) - world_x, (player_y-120) - world_y);
				weapons.curr_frame = 19;
				Draw_BOB(&weapons, lpddsback);
				break;
						}
			case NEAST: {
				Set_Pos_BOB(&weapons, (player_x-10) - world_x, (player_y-110) - world_y);
				weapons.curr_frame = 19;
				Draw_BOB(&weapons, lpddsback);
				break;
						}
			case EAST: {
				Set_Pos_BOB(&weapons, (player_x+25) - world_x, (player_y-75) - world_y);
				weapons.curr_frame = 19;
				Draw_BOB(&weapons, lpddsback);
				break;
						}
			case SEAST: {
				Set_Pos_BOB(&weapons, (player_x+20) - world_x, (player_y-40) - world_y);
				weapons.curr_frame = 19;
				Draw_BOB(&weapons, lpddsback);
				break;
						}
		}
		player_firing = false;
	}
}
// finds slope between two points and returns direction of x0,y0 pointing to x1, y1
int Slope_Direction(int x0, int y0, int x1, int y1) {
	// computer change in x/y
	int dx = x0 - x1;
	int dy = y0 - y1;
	// if dy is 0
	if(dx == 0 && dy > 0)
		return NORTH;
	else if(dx == 0 && dy <= 0)
		return SOUTH;
	if(dy == 0 && dx > 0)
		return WEST;
	else if(dy == 0 && dx <= 0)
		return EAST;
	
	float slope = (float)dy/(float)dx;
	// find correct directions, based on 4 quadrants and slope size relative to 2.666, -2.666
	// 0.375, -0.375
	// upper right
	if(x1>x0 && y1<y0) {
		if(slope <= -2.41421)
			return NORTH;
		else if(slope > -.414214)
			return EAST;
		else
			return NEAST;
	}
	// upper left
	else if(x1<x0 && y1<y0) {
		if(slope >= 2.41421)
			return NORTH;
		else if(slope < .414214)
			return WEST;
		else
			return NWEST;
	}
	// lower left
	else if(x1<x0 && y1>y0) {
		if(slope <= -2.41421)
			return SOUTH;
		else if(slope > -.414214)
			return WEST;
		else
			return SWEST;
	}
	// lower right
	else if(x1>x0 && y1>y0) {
		if(slope >= 2.41421)
			return SOUTH;
		else if(slope < .414214)
			return EAST;
		else
			return SEAST;
	}

}
// checks if player collides with walls/objects
void Player_Collision() {
	// computer upper left and lower right corners of players "hitbox",
	// hitbox is around legs only, to simulate upper perspective view
	// finds corresponding tile the corner is within
	int x0 = ((world_x + (SCREEN_WIDTH >> 1) - 21) >> 6),
		y0 = ((world_y + (SCREEN_HEIGHT >> 1)) >> 6),
		x1 = ((world_x + (SCREEN_WIDTH >> 1) + 20) >> 6),
		y1 = ((world_y + (SCREEN_HEIGHT >> 1) + 50) >> 6);
	// represents hitbox around player feet/legs for game enemy collision
	int px0 = (world_x + (SCREEN_WIDTH >> 1) - 11),
		py0 = (world_y + (SCREEN_HEIGHT >> 1)),
		px1 = (world_x + (SCREEN_WIDTH >> 1) + 10),
		py1 = (world_y + (SCREEN_HEIGHT >> 1) + 30);

	// c1 represents upper left corner, true if in wall tile, false otherwise
	// c2-c4 follows rect corners going clockwise
	bool c1 = 0,
		 c2 = 0,
		 c3 = 0,
		 c4 = 0;
	// checks if corners present in wall tiles, based on overall world array of tiles
	if(world[current_level][y0][x0].type >= 0 && world[current_level][y0][x0].type<= 27)
		c1 = true;
	if(world[current_level][y0][x1].type >= 0 && world[current_level][y0][x1].type<= 27)
		c2 = true;
	if(world[current_level][y1][x1].type >= 0 && world[current_level][y1][x1].type<= 27)
		c3 = true;
	if(world[current_level][y1][x0].type >= 0 && world[current_level][y1][x0].type<= 27)
		c4 = true;
	// checks if player trying to move through enemies, checks map tiles 4 spaces around
	int startx = (player_x >> 6) - 4;
	int starty = (player_y >> 6) - 4;

	int endx = startx + 8;
	int endy = starty + 8;
	// check if out of bounds
	if(startx < 0)
		startx = 0;
	else if(startx > MAP_LENGTH_X - 1)
		startx = MAP_LENGTH_X - 1;
	if(starty < 0)
		starty = 0;
	else if(starty > MAP_LENGTH_Y - 1)
		starty = MAP_LENGTH_Y - 1;

	if(endx < 0)
		endx = 0;
	else if(endx > MAP_LENGTH_X - 1)
		endx = MAP_LENGTH_X - 1;
	if(endy < 0)
		endy = 0;
	else if(endy > MAP_LENGTH_Y - 1)
		endy = MAP_LENGTH_Y - 1;

	// cycles through chosen map tiles, checks each object if collsion could or has occured based on
	// four corner of hitbox	
	for(int y = starty; y < endy; y++) {
		for(int x = startx; x < endx; x++) {
			for(int n = 0; n < world[current_level][y][x].locallist.size(); n++) {
				GAME_OBJECT enemy = world[current_level][y][x].locallist[n];
				// checks for overlap for each corner
				if(px0 >= enemy.x0	&& px0 <= enemy.x1 && enemy.visible == true) {
					if(py0 >= (enemy.y0-30) && py0 <= enemy.y1) {
						c1 = true;
							
					}
				}
				if(px1 >= enemy.x0	&& px1 <= enemy.x1 && enemy.visible == true) {
					if(py0 >= (enemy.y0-30) && py0 <= enemy.y1) {
						c2 = true;
					}
				}
				if(px1 >= enemy.x0	&& px1 <= enemy.x1 && enemy.visible == true) {
					if(py1 >= (enemy.y0-30) && py1 <= enemy.y1) {
						c3 = true;
					}
				}
				if(px0 >= enemy.x0	&& px0 <= enemy.x1 && enemy.visible == true) {
					if(py1 >= (enemy.y0-30) && py1 <= enemy.y1) {
						c4 = true;
					}
				}
			}
		}
	}

	// checks if one side of player is against a wall, adjusts change in x and y accordingly
	if(c1 && c2) {
		world_y -=player_dy;
		if(c3 || c4)
			world_x -=player_dx;
	}
	else if(c3 && c4) {
		world_y -=player_dy;
		if(c1 || c2)
			world_x -=player_dx;
	}
	else if(c2 && c3) {
		world_x -=player_dx;
		if(c1 || c4)
			world_y -=player_dy;
	}
	else if(c1 && c4) {
		world_x -=player_dx;
		if(c2 || c3)
			world_y -=player_dy;
	}
	else {
		// checks single corners to prevent moving into walls
		// also allows player to move along walls while trying to move into the wall
		// so long as player is not in a corner
		if(keyboard_state[DIK_W]){
			if(c1 || c2)
				world_y -=player_dy;
		}
		else if(keyboard_state[DIK_S]){
			if(c3 || c4)
				world_y -=player_dy;
		}

		if(keyboard_state[DIK_A]){
			if(c1 || c4)
				world_x -=player_dx;
		}
		else if(keyboard_state[DIK_D]){
			if(c2 || c3)
				world_x -=player_dx;
		}
	}
}
// prevents enemies from moving into walls
void Object_Collision_Wall(GAME_OBJECT &gameobject) {
	// initialized where each corner of objects hitbox, if its in a wall tile
	int startx = (gameobject.x0 >> 6);
	int starty = (gameobject.y0 >> 6);
	int endx = (gameobject.x1 >> 6);
	int endy = (gameobject.y1 >> 6);

	// c1 - c4 represents for corners of hitbox, starting at upper left and going clockwise
	bool c1 = 0,
		 c2 = 0,
		 c3 = 0,
		 c4 = 0;
	// checks if each corner is in a wall tile
	if(world[current_level][starty][startx].type >= 0 && world[current_level][starty][startx].type<= 27)
		c1 = true;
	if(world[current_level][starty][endx].type >= 0 && world[current_level][starty][endx].type<= 27)
		c2 = true;
	if(world[current_level][endy][endx].type >= 0 && world[current_level][endy][endx].type<= 27)
		c3 = true;
	if(world[current_level][endy][startx].type >= 0 && world[current_level][endy][startx].type<= 27)
		c4 = true;
	
	// checks if one side of enemy/object is against a wall, adjusts change in x and y accordingly
	// following checks if one side is moving up against a wall
	if(c1 && c2) {
		gameobject.y +=gameobject.dy;
		gameobject.y0 +=gameobject.dy;
		gameobject.y1 +=gameobject.dy;
		// prevents all movement if in a corner
		if(c4) {
			gameobject.x +=gameobject.dx;
			gameobject.x0 +=gameobject.dx;
			gameobject.x1 +=gameobject.dx;
		}
		else if(c3) {
			gameobject.x -=gameobject.dx;
			gameobject.x0 -=gameobject.dx;
			gameobject.x1 -=gameobject.dx;
		}
	}
	else if(c3 && c4) {
		gameobject.y -=gameobject.dy;
		gameobject.y0 -=gameobject.dy;
		gameobject.y1 -=gameobject.dy;
		if(c2) {
			gameobject.x -=gameobject.dx;
			gameobject.x0 -=gameobject.dx;
			gameobject.x1 -=gameobject.dx;
		}
		else if(c1) {
			gameobject.x +=gameobject.dx;
			gameobject.x0 +=gameobject.dx;
			gameobject.x1 +=gameobject.dx;
		}
	}
	else if(c2 && c3) {
		gameobject.x -=gameobject.dx;
		gameobject.x0 -=gameobject.dx;
		gameobject.x1 -=gameobject.dx;
		if(c1) {
			gameobject.y +=gameobject.dy;
			gameobject.y0 +=gameobject.dy;
			gameobject.y1 +=gameobject.dy;
		}
		else if(c4) {
			gameobject.y -=gameobject.dy;
			gameobject.y0 -=gameobject.dy;
			gameobject.y1 -=gameobject.dy;
		}
	}
	else if(c1 && c4) {
		gameobject.x +=gameobject.dx;
		gameobject.x0 +=gameobject.dx;
		gameobject.x1 +=gameobject.dx;
		if(c2) {
			gameobject.y +=gameobject.dy;
			gameobject.y0 +=gameobject.dy;
			gameobject.y1 +=gameobject.dy;
		}
		else if(c3) {
			gameobject.y -=gameobject.dy;
			gameobject.y0 -=gameobject.dy;
			gameobject.y1 -=gameobject.dy;
		}
	}
	else {
		// checks single corners to prevent moving into walls
		// also allows object to move along walls while trying to move into the wall
		// so long as player is not in a corner
		if(gameobject.direction >= NWEST && gameobject.direction <= NEAST){
			if(c1 || c2) {
				gameobject.y +=gameobject.dy;
				gameobject.y0 +=gameobject.dy;
				gameobject.y1 +=gameobject.dy;
			}
		}
		else if(gameobject.direction == SWEST || gameobject.direction == SOUTH || gameobject.direction == SEAST){
			if(c3 || c4) {
				gameobject.y -=gameobject.dy;
				gameobject.y0 -=gameobject.dy;
				gameobject.y1 -=gameobject.dy;
			}
		}

		if(gameobject.direction >= SWEST && gameobject.direction <= NWEST ){
			if(c1 || c4) {
				gameobject.x +=gameobject.dx;
				gameobject.x0 +=gameobject.dx;
				gameobject.x1 +=gameobject.dx;
			}
		}
		else if(gameobject.direction >= NEAST && gameobject.direction <= SEAST){
			if(c2 || c3) {
				gameobject.x -=gameobject.dx;
				gameobject.x0 -=gameobject.dx;
				gameobject.x1 -=gameobject.dx;
			}
		}
	}
}
// prevents enemies/objects from moving into eachother or overlapping
int Object_Collision_Object(int x0, int y0, int x1, int y1, int direction, int velocity) {

	GAME_OBJECT gameobject;
	// initialized current coord. of objects hitbox
	int nx0 = x0,
		ny0 = y0,
		nx1 = x1,
		ny1 = y1;
	// obtains current map tile each hitbox corner is located in
	int startx = (x0 >> 6),
	    starty = (y0 >> 6),
		endx = (x1 >> 6),
		endy = (y1 >> 6);
	// checks which direction object is moving, update where object will be
	if(direction >= NEAST && direction <= SEAST) {
		nx0 += velocity;
		nx1 += velocity;
	}
	else if(direction >= SWEST && direction <= NWEST){
		nx0 -= velocity;
		nx1 -= velocity;
	}
	if(direction >= NWEST && direction <= NEAST) {
		ny0 -= velocity;
		ny1 -= velocity;
	}
	else if(direction == SWEST || direction == SOUTH || direction == SEAST){
		ny0 += velocity;
		ny1 += velocity;
	}
	// setup area to search around object, 4 spaces move around however big the objects hitbox is
	// and checks for out of bounds
	startx -= 4;
	starty -= 4;
	if(startx < 0)
		startx = 0;
	else if(startx > MAP_LENGTH_X - 1)
		startx = MAP_LENGTH_X - 1;
	if(starty < 0)
		starty = 0;
	else if(starty > MAP_LENGTH_Y - 1)
		starty = MAP_LENGTH_Y - 1;

	endx += 4;
	endy += 4;
	if(endx < 0)
		endx = 0;
	else if(endx > MAP_LENGTH_X - 1)
		endx = MAP_LENGTH_X - 1;
	if(endy < 0)
		endy = 0;
	else if(endy > MAP_LENGTH_Y - 1)
		endy = MAP_LENGTH_Y - 1;
	// checks objects in all tiles surrounding object, if object will overlap by moving, returns false
	// meaning collision occurred, used function rect_collision to check
	for(int y = starty; y < endy; y++) {
		for(int x = startx; x < endx; x++) {
			for(int index = 0; index < world[current_level][y][x].locallist.size(); index++) {
				gameobject = world[current_level][y][x].locallist[index];
				if((gameobject.x0 != x0 && gameobject.y0 != y0) &&
						(gameobject.x1 != x1 && gameobject.y1 != y1) && gameobject.visible == true) {
					if(Rect_Collision(nx0, ny0, nx1, ny1, 
										gameobject.x0, gameobject.y0, gameobject.x1, gameobject.y1))
						return 0;
				}
			}
		}
	}
	// no collisions, object is free to move, returns true
	return 1;
}
// checks for overlap, return true if so, false if not collision
int Rect_Collision(int x0,int y0,int x1,int y1,int x2,int y2,int x3,int y3) {
	if((x0 >= x2 && x0 <= x3) && (y0 >= y2 && y0 <= y3))
		return 1;
	if((x0 >= x2 && x0 <= x3) && (y1 >= y2 && y1 <= y3))
		return 1;
	if((x1 >= x2 && x1 <= x3) && (y0 >= y2 && y0 <= y3))
		return 1;
	if((x1 >= x2 && x1 <= x3) && (y1 >= y2 && y1 <= y3))
		return 1;

	return 0;
}

// computes AI of all game objects on screen and a few tiles off screen
void Game_AI() {

	int index, index_x, index_y;  // looping vars
	int start_map_x,start_map_y,  // map positions
		end_map_x,end_map_y; 

	// compute start of map rectangle for best cast i.e. aligned on 64x64 boundary
	start_map_x = (world_x >> 6) - 4; // use >> 6 for speed, but this is clearer
	start_map_y = (world_y >> 6) - 2; 
	// if start of map is out of bounds of world matrix
	if(start_map_x < 0)
		start_map_x = 0;
	else if(start_map_x > MAP_LENGTH_X - 1)
		start_map_x = MAP_LENGTH_X - 1;
	if(start_map_y < 0)
		start_map_y = 0;
	else if(start_map_y > MAP_LENGTH_Y - 1)
		start_map_y = MAP_LENGTH_Y - 1;
	// compute end of map rectangle for best cast i.e. aligned on 64x64 boundary
	end_map_x = start_map_x + (SCREEN_WIDTH  >> 6) + 5;
	end_map_y = start_map_y + (SCREEN_HEIGHT >> 6) + 4;
	// if end of map is out of bounds of world matrix
	if(end_map_x < 0)
		end_map_x = 0;
	else if(end_map_x > MAP_LENGTH_X - 1)
		end_map_x = MAP_LENGTH_X - 1;
	if(end_map_y < 0)
		end_map_y = 0;
	else if(end_map_y > MAP_LENGTH_Y - 1)
		end_map_y = MAP_LENGTH_Y - 1;
	// checks all map tiles displayed on screen, plus some off screen to computer AI for all objects
	for (index_y = start_map_y; index_y < end_map_y; index_y++)
		{
		for (index_x = start_map_x; index_x < end_map_x; index_x++)
			{
				// checks each vector object
				for(index = 0; index < world[current_level][index_y][index_x].locallist.size(); index++)
				{	// checks for appropiate AI function to enter for right object

					switch(world[current_level][index_y][index_x].locallist[index].type) {
						case ZOMBIE: {
							Zombie_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
									 }
						case BLOB: {
							Blob_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
									 }
						case SPIDER_BLOB: {
							Spider_Blob_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
									 }
						case BEHEMOTH: {
							Behemoth_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
									 }
						case MACHINE_GUN: {
							Machine_Gun_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case ENERGY_GUN: {
							Energy_Gun_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case FLAME_GUN: {
							Flame_Gun_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case ENERGYAMMO_ITEM: {
							Item_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case MACHINEAMMO_ITEM: {
							Item_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case HP_ITEM: {
							Item_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case FLAMEAMMO_ITEM: {
							Item_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case BARREL_ITEM: {
							Item_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case KEY_ITEM: {
							Item_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case LARGE_EXPLOSION: {
							Explosion_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }
						case SMALL_EXPLOSION: {
							Explosion_AI(world[current_level][index_y][index_x].locallist[index], index);
							break;
										  }

						default: break;
					}

				}

			} // end for map_x

	} // end for map_y

}
// computes appropiate AI for zombie object, most other enemie AI are modified versions of
// the zombie
void Zombie_AI(GAME_OBJECT &gameobject, int index) {
	// for new and old map positions, if entering an new map tile's list
	int new_x, new_y, temp1, temp2;
	// value if objects should move or not
	int move;
	// for distance between zombie and player
	int distance;
	// if object has died
	if(gameobject.hp <= 0 && gameobject.alive) {
		DSound_Play(sounds[11]);			
		gameobject.visible = false;
		gameobject.alive = false;
		gameobject.delay = 0;
		// load death animation/frame
		gameobject.curr_frame = 0;
		gameobject.direction = DEATH;
		// 25% chance that zombie's death creates a blob enemy, init blob, place into appropiate map tile list
		if((rand() % 7) == 0) {
			GAME_OBJECT newgameobject;
			newgameobject.type = BLOB;
			newgameobject.x = gameobject.x;
			newgameobject.y = gameobject.y;
			newgameobject.wx = (gameobject.x >> 6);
			newgameobject.wy = (gameobject.y >> 6);
			temp1 = newgameobject.wx;
			temp2 = newgameobject.wy;
			newgameobject.direction = rand() % 8;
			// hit box of object centered around legs/feet
			newgameobject.x0 = newgameobject.x;
			newgameobject.y0 = newgameobject.y;
			newgameobject.x1 = newgameobject.x + 39;
			newgameobject.y1 = newgameobject.y + 45;
			newgameobject.anim_max = 4;
			newgameobject.curr_frame = 0;
			newgameobject.dx = 2;
			newgameobject.dy = 2;
			newgameobject.delay = 0;
			newgameobject.commit = 30;
			newgameobject.stuck = 0;
			newgameobject.visible = true;
			newgameobject.hit = false;
			newgameobject.alive = true;
			newgameobject.hp = 100;
			newgameobject.attack = 0;
			newgameobject.special1 = 0;
			world[current_level][temp2][temp1].locallist.push_back(newgameobject);
		}
	}
	if(gameobject.alive) {
		// plays zombie sound
		if(DSound_Status_Sound(sounds[10]) != DSBSTATUS_PLAYING)
			DSound_Play(sounds[10]);
		// if hit, knocks zombie back some, else normal movement
		if(gameobject.hit == true) {
			gameobject.hit = false;
			gameobject.dx = -10;
			gameobject.dy = -10;
		}
		else {
			gameobject.dx = 2;
			gameobject.dy = 2;
		}
		// if commit is 0, zombie reevaluates movement direction towards player
		if(gameobject.commit == 0) {
			// sets commitment, prevents enemy continously moving erratically if zombie is 
			// moving in a line on the threshold between to direction of the main 8
			gameobject.commit = 30;
			// find direction towards player, resets animation if differant
			temp1 = Slope_Direction(gameobject.x + 50, gameobject.y + 85, player_x, player_y);
			if(temp1 != gameobject.direction) {
				gameobject.direction = temp1;
				gameobject.curr_frame = 0;
				gameobject.delay = 0;
			}
		}
		else
			gameobject.commit--;
		// loads new animation, objects delay prevents switching of frames too fast
		// cycle through frames as well, usually about 4 for movement/attack/death
		if(gameobject.delay < zombie.anim_count_max)
			gameobject.delay++;
		else {
			gameobject.delay = 0;
			if(gameobject.curr_frame < 3)
				gameobject.curr_frame++;
			else
				gameobject.curr_frame = 0;
		}
		if(gameobject.curr_frame >= 4)
			gameobject.curr_frame = 3;
		if(gameobject.direction >= 17)
			gameobject.direction = 16;
		// checks collision with wall
		Object_Collision_Wall(gameobject);
		// checks collision with another object, move = 1 if yes, 0 no
		move = Object_Collision_Object(gameobject.x0, gameobject.y0, gameobject.x1, gameobject.y1,
											gameobject.direction, gameobject.dx);
		if(move == 1) {
			//check distance to player
			distance = Fast_Distance_2D((gameobject.x + 50 - player_x), (gameobject.y + 85 - player_y));
			// resets stuck, no object stuck together
			gameobject.stuck = 0;
			// object can now be checked again for collision
			gameobject.visible = true;
			// zombie attack player, if distance < 40
			if(distance <= 40) {
				// reset commit, in order to load new animation
				gameobject.commit = 0;
				// set zombie BOB, and draw
				Set_Pos_BOB(&zombie, gameobject.x - world_x, gameobject.y - world_y);
				zombie.curr_frame = anims[gameobject.direction + 8][gameobject.curr_frame];
				Draw_BOB(&zombie, lpddsback);
				// checks attack counter, prevents zombie attack every frame of game, harms player, makes sound
				if(gameobject.attack == 0) {
					DSound_Play(sounds[5]);
					player_health -= 3;
					gameobject.attack = 30;
				}
				else
					gameobject.attack -= 1;
			}
			else {
				// if not attacking, moves zombie in direction chosen
				if(gameobject.direction >= NEAST && gameobject.direction <= SEAST) {
					gameobject.x += gameobject.dx;
					gameobject.x0 += gameobject.dx;
					gameobject.x1 += gameobject.dx;
				}
				else if(gameobject.direction >= SWEST && gameobject.direction <= NWEST){
					gameobject.x -= gameobject.dx;
					gameobject.x0 -= gameobject.dx;
					gameobject.x1 -= gameobject.dx;
				}
				if(gameobject.direction >= NWEST && gameobject.direction <= NEAST) {
					gameobject.y -= gameobject.dy;
					gameobject.y0 -= gameobject.dy;
					gameobject.y1 -= gameobject.dy;
				}
				else if(gameobject.direction == SWEST || gameobject.direction == SOUTH || gameobject.direction == SEAST){
					gameobject.y += gameobject.dy;
					gameobject.y0 += gameobject.dy;
					gameobject.y1 += gameobject.dy;
				}
				// checks if zombie entered new map tile, delete from old tile list, adds to new tile list
				// sorts objects, prevent inappropiate overlap drawing
				new_x = (gameobject.x0 >> 6);
				new_y = (gameobject.y1 >> 6);
				if((new_x != gameobject.wx) || (new_y != gameobject.wy)) {
					std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
					temp += index;
					temp1 = gameobject.wx;
					temp2 = gameobject.wy;
					gameobject.wx = new_x;
					gameobject.wy = new_y;
					world[current_level][new_y][new_x].locallist.push_back(gameobject);
					world[current_level][temp2][temp1].locallist.erase(temp);
					std::sort(world[current_level][new_y][new_x].locallist.begin(),
								world[current_level][new_y][new_x].locallist.end(), Sort_Objects);
				}
			}
		}
		else {
			// if collision with object, increments stuck, chooses new direction (bounce off)
			// prevents all zombie from becoming to close to following a single line towards player
			// sets new commit, keeps object moving in direction a little longer
			gameobject.stuck += 1;
			gameobject.direction = rand()%8;
			gameobject.curr_frame = 0;
			gameobject.delay = 0;
			gameobject.commit = 30;
		}
		if(gameobject.curr_frame >= 4)
			gameobject.curr_frame = 3;
		if(gameobject.direction >= 17)
			gameobject.direction = 16;
		// draws zombie if visible and not attacking
		if(gameobject.visible && distance > 40) {
			Set_Pos_BOB(&zombie, gameobject.x - world_x, gameobject.y - world_y);
			zombie.curr_frame = anims[gameobject.direction][gameobject.curr_frame];
			Draw_BOB(&zombie, lpddsback);
		}
		// if zombie is stuck long enough, sets visible to false, object will not be checked for collision
		// until theres room to move again
		if(gameobject.stuck == 3)
			gameobject.visible = false;
	}
	else {
		// cycles through 4 frame of death animation
		if(gameobject.curr_frame != 3) {
			// increments delay, prevent rapid processing of death animation
			if(gameobject.delay < zombie.anim_count_max + 5)
				gameobject.delay++;
			else {
				// increment death animation
				gameobject.delay = 0;
				gameobject.curr_frame++;
				// makes zombie fall forward looking,.. draws object down further on screen
				// prevents dead object from overlapping other objects based on their size
				if(gameobject.curr_frame == 2)
					gameobject.y += 80;
			}
		}
		// draws death animation, only if player chooses to have violence displayed, using 'v' key
		if(violence == true) {
			Set_Pos_BOB(&zombie, gameobject.x - world_x, gameobject.y - world_y);
			zombie.curr_frame = anims[gameobject.direction][gameobject.curr_frame];
			Draw_BOB(&zombie, lpddsback);
		}
	}
}

// computes appropiate AI for blob object, 
// refer to Zombie_AI function for comments where none of present here
void Blob_AI(GAME_OBJECT &gameobject, int index) {
	int new_x, new_y, temp1, temp2;
	int move;
	int distance;
	// blob dies if heatlh <= 65 instead of 0, prevents drawing scaled version that is too small to
	// see and hit,
	if(gameobject.hp <= 65 && gameobject.alive) {
			gameobject.visible = false;
			gameobject.alive = false;
			gameobject.delay = 0;
			gameobject.curr_frame = 16;
	}
	if(gameobject.alive) {
		// play blob sound, if not currently running
		if(DSound_Status_Sound(sounds[15]) != DSBSTATUS_PLAYING)
			DSound_Play(sounds[15]);
		// gameobject offsets create point centered in blob image
		distance = Fast_Distance_2D(player_x - (gameobject.x + 19), player_y - (gameobject.y + 22));
		// resets default movement speed
		gameobject.dx = gameobject.dy = 2;
		// special 1 counter allows blob to increase in size, but not for every frame
		if(gameobject.special1 > 0)
			gameobject.special1--;
		// if hit by a weapon
		if(gameobject.hit == true) {
			// play sound
			DSound_Play(sounds[16]);
			gameobject.hit = false;
			// blob will move in direction of play no matter what
			gameobject.direction = Slope_Direction(gameobject.x + 19, gameobject.y + 22, player_x, player_y);
			gameobject.commit = 30;
			// weapons other than flame thrower increased blobs hp and size, 200 is max, 200 changes blob to
			// blob spider
			if(gameobject.hp >= 200) {
				gameobject.hp = 200;
			}
			// updates blob size based on hp by increased hp change over based on differance between 100
			// and actual hp, shrinks or grows in relation to differance from 100 hp
			if(gameobject.special1 == 0) {
				gameobject.special1 = 10;
				temp1 = ((gameobject.hp - 100) >> 1);
				gameobject.x0 = gameobject.x - temp1;
				gameobject.y0 = gameobject.y - temp1;
				gameobject.x1 = gameobject.x + 39 + temp1;
				gameobject.y1 = gameobject.y + 45 + temp1;
			}
		}
		else {
			// normal movement speed
			gameobject.dx = 1;
			gameobject.dy = 1;
		}

		if(gameobject.commit == 0) {
			// moves towards player only if within this distance
			if(distance <= 250) {
				gameobject.direction = Slope_Direction(gameobject.x + 19, gameobject.y + 22, player_x, player_y);
				gameobject.dx = gameobject.dy = 1;
			}
			else {
				// if not near player, moves randomly
				gameobject.commit = 15;
				gameobject.direction = rand() % 8;
			}
		}
		else
			gameobject.commit--;

		if(gameobject.delay < blob.anim_count_max)
			gameobject.delay++;
		else {
			gameobject.delay = 0;
			if(gameobject.curr_frame < 15)
				gameobject.curr_frame++;
			else
				gameobject.curr_frame = 0;
		}
		
		Object_Collision_Wall(gameobject);
		move = Object_Collision_Object(gameobject.x0, gameobject.y0, gameobject.x1, gameobject.y1,
											gameobject.direction, gameobject.dx);
		if(move == 1) {
			gameobject.stuck = 0;
			gameobject.visible = true;
			// distance in order to attack
			if(distance <= 50) {
				Set_Pos_BOB(&blob, gameobject.x0 - world_x, gameobject.y0 - world_y);
				blob.curr_frame = gameobject.curr_frame;
				// draws blob based on hp differance from 100 default hp, uses bounding hitbox rectangle to
				// draw
				Draw_Scaled_BOB(&blob, gameobject.x1 - gameobject.x0, gameobject.y1 - gameobject.y0,lpddsback);
				if(gameobject.attack == 0) {
					DSound_Play(sounds[5]);
					player_health -= 6;
					gameobject.attack = 30;
				}
				else
					gameobject.attack -= 1;
			}
			else {
				if(gameobject.direction >= NEAST && gameobject.direction <= SEAST) {
					gameobject.x += gameobject.dx;
					gameobject.x0 += gameobject.dx;
					gameobject.x1 += gameobject.dx;
				}
				else if(gameobject.direction >= SWEST && gameobject.direction <= NWEST){
					gameobject.x -= gameobject.dx;
					gameobject.x0 -= gameobject.dx;
					gameobject.x1 -= gameobject.dx;
				}
				if(gameobject.direction >= NWEST && gameobject.direction <= NEAST) {
					gameobject.y -= gameobject.dy;
					gameobject.y0 -= gameobject.dy;
					gameobject.y1 -= gameobject.dy;
				}
				else if(gameobject.direction == SWEST || gameobject.direction == SOUTH || gameobject.direction == SEAST){
					gameobject.y += gameobject.dy;
					gameobject.y0 += gameobject.dy;
					gameobject.y1 += gameobject.dy;
				}
				new_x = (gameobject.x >> 6);
				new_y = (gameobject.y >> 6);
				if((new_x != gameobject.wx) || (new_y != gameobject.wy)) {
					std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
					temp += index;
					temp1 = gameobject.wx;
					temp2 = gameobject.wy;
					gameobject.wx = new_x;
					gameobject.wy = new_y;
					world[current_level][new_y][new_x].locallist.push_back(gameobject);
					world[current_level][temp2][temp1].locallist.erase(temp);
					std::sort(world[current_level][new_y][new_x].locallist.begin(),
								world[current_level][new_y][new_x].locallist.end(), Sort_Objects);
				}
			}
		}
		else {
			gameobject.stuck += 1;
			gameobject.direction = rand()%8;
			gameobject.delay = 0;
			gameobject.commit = 30;
		}
		// draw if not attacking
		if(distance > 50) {
			Set_Pos_BOB(&blob, gameobject.x0 - world_x, gameobject.y0 - world_y);
			blob.curr_frame = gameobject.curr_frame;
			Draw_Scaled_BOB(&blob, gameobject.x1 - gameobject.x0, gameobject.y1 - gameobject.y0,lpddsback);
		}

		if(gameobject.stuck == 3)
			gameobject.visible = false;
	}
	// just deletes blob if dead, no death animation
	else {
		std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
		temp += index;
		temp1 = gameobject.wx;
		temp2 = gameobject.wy;
		world[current_level][temp2][temp1].locallist.erase(temp);
	}
	// if hp 200,deletes blob and replaces with a blob spider
	// inits spider based on location of blob
	if(gameobject.hp == 200) {
		DSound_Play(sounds[18]);
		std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
					temp += index;
		new_x = (gameobject.x >> 6);
		new_y = (gameobject.y >> 6);
		gameobject.type = SPIDER_BLOB;
		gameobject.direction = rand() % 8;
		// hit box of object centered around legs/feet
		gameobject.x0 = gameobject.x;
		gameobject.y0 = gameobject.y + 30;
		gameobject.x1 = gameobject.x0 + 160;
		gameobject.y1 = gameobject.y0 + 120;
		gameobject.wx = temp1 = (gameobject.x0 >> 6);
		gameobject.wy = temp2 = (gameobject.y1 >> 6);
		gameobject.anim_max = 4;
		gameobject.curr_frame = 0;
		gameobject.dx = 2;
		gameobject.dy = 2;
		gameobject.delay = 0;
		gameobject.commit = 0;
		gameobject.stuck = 0;
		gameobject.visible = true;
		gameobject.hit = false;
		gameobject.alive = true;
		gameobject.hp = 100;
		gameobject.attack = 0;
		gameobject.special1 = 300;
		gameobject.special2 = 10;
		world[current_level][temp2][temp1].locallist.push_back(gameobject);
		world[current_level][new_y][new_x].locallist.erase(temp);
		std::sort(world[current_level][temp2][temp1].locallist.begin(),
						world[current_level][temp2][temp1].locallist.end(), Sort_Objects);
	}
}

// computes appropiate AI for blob spider object, 
// refer to Zombie_AI function for comments where none of present here
void Spider_Blob_AI(GAME_OBJECT &gameobject, int index) {
	int new_x, new_y, temp1, temp2;
	int move;
	int distance, distance_target; // distance_target, is dist from center of spider to mouse target

	if(gameobject.hp <= 0 && gameobject.alive) {
		// stops crawling sound, will begin again if another spider is present
		DSound_Stop_Sound(sounds[20]);
		DSound_Play(sounds[17]);
		gameobject.visible = false;
		gameobject.alive = false;
		gameobject.delay = 0;
		gameobject.curr_frame = 0;
		gameobject.direction = DEATH;
	}

	if(gameobject.alive) {
		if(DSound_Status_Sound(sounds[20]) != DSBSTATUS_PLAYING)
			DSound_Play(sounds[20]);
		// special2 is init. with 10, meaning spider blob can create 10 blobs over a time
		// based on the counter special1
		if(gameobject.special2 > 0) {
			// special1 == 0 allows the creation of a new blob, init.s blob based on spider's location
			// and loads new blob into correct map tile list
			if(gameobject.special1 == 0) {
				gameobject.special2--;
				gameobject.special1 = 150;
				GAME_OBJECT newgameobject;
				newgameobject.type = BLOB;
				newgameobject.x = gameobject.x;
				newgameobject.y = gameobject.y;
				newgameobject.wx = (gameobject.x >> 6);
				newgameobject.wy = (gameobject.y >> 6);
				temp1 = newgameobject.wx;
				temp2 = newgameobject.wy;
				newgameobject.direction = rand() % 8;
				// hit box of object centered around legs/feet
				newgameobject.x0 = newgameobject.x + 30;
				newgameobject.y0 = newgameobject.y;
				newgameobject.x1 = newgameobject.x0 + 80;
				newgameobject.y1 = newgameobject.y + 45;
				newgameobject.anim_max = 4;
				newgameobject.curr_frame = 0;
				newgameobject.dx = 2;
				newgameobject.dy = 2;
				newgameobject.delay = 0;
				newgameobject.commit = 30;
				newgameobject.stuck = 0;
				newgameobject.visible = true;
				newgameobject.hit = false;
				newgameobject.alive = true;
				newgameobject.hp = 100;
				newgameobject.attack = 0;
				newgameobject.special1 = 0;
				world[current_level][temp2][temp1].locallist.push_back(newgameobject);
			}
			else
				gameobject.special1--;
		}
		// if hit by weapon, knocks spider back
		if(gameobject.hit == true) {
			gameobject.hit = false;
			gameobject.dx = -10;
			gameobject.dy = -10;
		}
		else {
			// normal spider speed
			gameobject.dx = 7;
			gameobject.dy = 7;
		}
		if(gameobject.commit == 0) {
			gameobject.commit = 5;
			// dist. 200 threshold for spider to move in opposite of direction from mouse target
			// in effect, runs away when player aims for spider
			distance_target = Fast_Distance_2D(gameobject.x + 80-(world_x + mouse_x), gameobject.y + 80-(world_y + mouse_y));
			if(distance_target <= 200) {
				gameobject.commit = 5;
				// speed when running away
				gameobject.dx = 10;
				gameobject.dy = 10;
				// finds direction from spider to target
				temp1 = (Slope_Direction(gameobject.x + 80, gameobject.y + 80, world_x + mouse_x, world_y + mouse_y) + 4);
				// finds opposite direction to run away
				if(temp1 > 7)
					temp1 = (temp1 % 7) - 1;
			}
			else // finds direction to player
				temp1 = Slope_Direction(gameobject.x + 80, gameobject.y + 80, player_x, player_y);
			if(temp1 != gameobject.direction) {
				gameobject.direction = temp1;
				gameobject.curr_frame = 0;
				gameobject.delay = 0;
			}
		}
		else
			gameobject.commit--;

		if(gameobject.delay < spider_blob.anim_count_max)
			gameobject.delay++;
		else {
			gameobject.delay = 0;
			if(gameobject.curr_frame < 3)
				gameobject.curr_frame++;
			else
				gameobject.curr_frame = 0;
		}

		if(gameobject.curr_frame >= 4)
			gameobject.curr_frame = 3;
		if(gameobject.direction >= 17)
			gameobject.direction = 16;

		Object_Collision_Wall(gameobject);
		/*move = Object_Collision_Object(gameobject.x0, gameobject.y0, gameobject.x1, gameobject.y1,
											gameobject.direction, gameobject.dx);*/
		move = 1;
		if(move == 1) {
			distance = Fast_Distance_2D((gameobject.x + 80 - player_x), (gameobject.y + 80 - player_y));
			gameobject.stuck = 0;
			gameobject.visible = true;
			// dist. 120 threshold to attack player, attack distance based on size of enemy,
			// taking into account center of enemy image to player coords.
			if(distance <= 120) {
				if(DSound_Status_Sound(sounds[19]) != DSBSTATUS_PLAYING)
					DSound_Play(sounds[19]);
				Set_Pos_BOB(&spider_blob, gameobject.x - world_x, gameobject.y - world_y);
				spider_blob.curr_frame = anims[gameobject.direction + 8][gameobject.curr_frame];
				Draw_BOB(&spider_blob, lpddsback);
				if(gameobject.attack == 0) {
					DSound_Play(sounds[5]);
					player_health -= 15;
					gameobject.attack = 30;
				}
				else
					gameobject.attack -= 1;
			}
			else {
				if(gameobject.direction >= NEAST && gameobject.direction <= SEAST) {
					gameobject.x += gameobject.dx;
					gameobject.x0 += gameobject.dx;
					gameobject.x1 += gameobject.dx;
				}
				else if(gameobject.direction >= SWEST && gameobject.direction <= NWEST){
					gameobject.x -= gameobject.dx;
					gameobject.x0 -= gameobject.dx;
					gameobject.x1 -= gameobject.dx;
				}
				if(gameobject.direction >= NWEST && gameobject.direction <= NEAST) {
					gameobject.y -= gameobject.dy;
					gameobject.y0 -= gameobject.dy;
					gameobject.y1 -= gameobject.dy;
				}
				else if(gameobject.direction == SWEST || gameobject.direction == SOUTH || gameobject.direction == SEAST){
					gameobject.y += gameobject.dy;
					gameobject.y0 += gameobject.dy;
					gameobject.y1 += gameobject.dy;
				}
				new_x = (gameobject.x0 >> 6);
				new_y = (gameobject.y1 >> 6);
				if((new_x != gameobject.wx) || (new_y != gameobject.wy)) {
					std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
					temp += index;
					temp1 = gameobject.wx;
					temp2 = gameobject.wy;
					gameobject.wx = new_x;
					gameobject.wy = new_y;
					world[current_level][new_y][new_x].locallist.push_back(gameobject);
					world[current_level][temp2][temp1].locallist.erase(temp);
					std::sort(world[current_level][new_y][new_x].locallist.begin(),
								world[current_level][new_y][new_x].locallist.end(), Sort_Objects);
				}
			}
		}
		else {
			gameobject.stuck += 1;
			gameobject.direction = rand()%8;
			gameobject.curr_frame = 0;
			gameobject.delay = 0;
			gameobject.commit = 10;
		}

		if(gameobject.visible == true && distance > 120) {
			Set_Pos_BOB(&spider_blob, gameobject.x - world_x, gameobject.y - world_y);
			spider_blob.curr_frame = anims[gameobject.direction][gameobject.curr_frame];
			Draw_BOB(&spider_blob, lpddsback);
		}
		
		if(gameobject.stuck == 5)
			gameobject.visible = false;
	}
	else {
		if(gameobject.curr_frame != 3) {
			if(gameobject.delay < spider_blob.anim_count_max + 5)
				gameobject.delay++;
			else {
				gameobject.delay = 0;
				gameobject.curr_frame++;
			}
		}
		if(violence == true) {
			Set_Pos_BOB(&spider_blob, gameobject.x - world_x, gameobject.y - world_y);
			spider_blob.curr_frame = anims[gameobject.direction][gameobject.curr_frame];
			Draw_BOB(&spider_blob, lpddsback);
		}
	}
}

// computes appropiate AI for behemoth enemy object, 
// refer to Zombie_AI function for comments where none of present here
void Behemoth_AI(GAME_OBJECT &gameobject, int index) {
	int new_x, new_y, temp1, temp2;
	int move;
	int distance;

	if(gameobject.hp <= 0 && gameobject.alive) {
			DSound_Play(sounds[12]);
			gameobject.visible = false;
			gameobject.alive = false;
			gameobject.delay = 0;
			gameobject.curr_frame = 0;
			gameobject.direction = DEATH;
	}

	if(gameobject.alive) {
		if(DSound_Status_Sound(sounds[13]) != DSBSTATUS_PLAYING)
				DSound_Play(sounds[13]);
		// if behemoth not in running mode, 151-500 range is normal walking
		if(gameobject.special1 > 150) {
			gameobject.dx = 2;
			gameobject.dy = 2;
		}
		else {
			// running speed
			gameobject.dx = 6;
			gameobject.dy = 6;
		}
		// if time for behemoth to run again, resets special1, 150 commit commits enemy to run 
		// in a straight line, unless collision with player
		if(gameobject.special1 == 500) {
				gameobject.special1 = 0;
				gameobject.commit = 150;
		}
		else
			gameobject.special1++;

		if(gameobject.commit == 0) {
			gameobject.commit = 30;
			temp1 = Slope_Direction(gameobject.x + 115, gameobject.y + 170, player_x, player_y);
			if(temp1 != gameobject.direction) {
				gameobject.direction = temp1;
				gameobject.curr_frame = 0;
				gameobject.delay = 0;
			}
		}
		else
			gameobject.commit--;

		if(gameobject.delay < behemoth.anim_count_max)
			gameobject.delay++;
		else {
			gameobject.delay = 0;
			if(gameobject.curr_frame < 3)
				gameobject.curr_frame++;
			else
				gameobject.curr_frame = 0;
		}

		if(gameobject.curr_frame >= 4)
			gameobject.curr_frame = 3;
		if(gameobject.direction >= 17)
			gameobject.direction = 16;

		// checks each corner of hitbox for juggernaut, upper left first and the rest clockwise
		// if corner is in a wall, turns wall tile to rubble, and creates explosion objects other the tile
		// breaks through walls, and allows other object enemies to move through
		// temp1 holds map tile type
		temp1 = world[current_level][gameobject.y0 >> 6][gameobject.x0 >> 6].type;
		// if map tile type is a wall type, index 0-27
		if(temp1 >= 0 && temp1 <= 27) {
			// play wall explosion sound
			DSound_Play(sounds[3]);
			// changes map tile to rubble
			world[current_level][(gameobject.y0 >> 6)][(gameobject.x0 >> 6)].type = 32;
			// init. wall explosion object based on current map tile location 
			GAME_OBJECT newgameobject;
			newgameobject.type = SMALL_EXPLOSION;
			newgameobject.visible = false;
			newgameobject.hit = false;
			newgameobject.x = gameobject.x1 - 295;
			newgameobject.y = gameobject.y0 - 45;
			newgameobject.y1 = 0;
			newgameobject.delay = 0;
			newgameobject.curr_frame = 11;
			newgameobject.wx = (newgameobject.x >> 6);
			newgameobject.wy = (newgameobject.y >> 6) + 2;
			// checks out of bounds of world array
			if(newgameobject.wx < 0)
				newgameobject.wx = 0;
			if(newgameobject.wy >= MAP_LENGTH_Y - 1)
				newgameobject.wy = MAP_LENGTH_Y - 2;
			temp1 = newgameobject.wx;
			temp2 = newgameobject.wy;
			Fix_Bounds(temp1, temp2);
			world[current_level][temp2][temp1].locallist.push_back(newgameobject);
		}
		// checks upper right corner
		temp1 = world[current_level][(gameobject.y0 >> 6)][(gameobject.x1 >> 6)].type;
		if(temp1 >= 0 && temp1 <= 27) {
			DSound_Play(sounds[3]);
			world[current_level][(gameobject.y0 >> 6)][(gameobject.x1 >> 6)].type = 32;
			GAME_OBJECT newgameobject;
			newgameobject.type = SMALL_EXPLOSION;
			newgameobject.visible = false;
			newgameobject.hit = false;
			newgameobject.x = gameobject.x1 - 45;
			newgameobject.y = gameobject.y0 - 45;
			newgameobject.y1 = 0;
			newgameobject.delay = 0;
			newgameobject.curr_frame = 11;
			newgameobject.wx = (newgameobject.x >> 6);
			newgameobject.wy = (newgameobject.y >> 6) + 2;
			if(newgameobject.wx < 0)
				newgameobject.wx = 0;
			if(newgameobject.wy >= MAP_LENGTH_Y - 1)
				newgameobject.wy = MAP_LENGTH_Y - 2;
			temp1 = newgameobject.wx;
			temp2 = newgameobject.wy;
			world[current_level][temp2][temp1].locallist.push_back(newgameobject);
		}
		// checks lower right corner
		temp1 = world[current_level][(gameobject.y1 >> 6)][(gameobject.x1 >> 6)].type;
		if(temp1 >= 0 && temp1 <= 27) {
			DSound_Play(sounds[3]);
			world[current_level][(gameobject.y1 >> 6)][(gameobject.x1 >> 6)].type = 32;
			GAME_OBJECT newgameobject;
			newgameobject.type = SMALL_EXPLOSION;
			newgameobject.visible = false;
			newgameobject.hit = false;
			newgameobject.x = gameobject.x1 - 45;
			newgameobject.y = gameobject.y1 - 45;
			newgameobject.y1 = 0;
			newgameobject.delay = 0;
			newgameobject.curr_frame = 11;
			newgameobject.wx = (newgameobject.x >> 6);
			newgameobject.wy = (newgameobject.y >> 6) + 2;
			if(newgameobject.wx < 0)
				newgameobject.wx = 0;
			if(newgameobject.wy >= MAP_LENGTH_Y - 1)
				newgameobject.wy = MAP_LENGTH_Y - 2;
			temp1 = newgameobject.wx;
			temp2 = newgameobject.wy;
			world[current_level][temp2][temp1].locallist.push_back(newgameobject);
		}
		// checks lower left corner
		temp1 = world[current_level][(gameobject.y1 >> 6)][(gameobject.x0 >> 6)].type;
		if(temp1 >= 0 && temp1 <= 27) {
			DSound_Play(sounds[3]);
			world[current_level][(gameobject.y1 >> 6)][(gameobject.x0 >> 6)].type = 32;
			GAME_OBJECT newgameobject;
			newgameobject.type = SMALL_EXPLOSION;
			newgameobject.visible = false;
			newgameobject.hit = false;
			newgameobject.x = gameobject.x0 - 45;
			newgameobject.y = gameobject.y1 - 45;
			newgameobject.y1 = 0;
			newgameobject.delay = 0;
			newgameobject.curr_frame = 11;
			newgameobject.wx = (newgameobject.x >> 6);
			newgameobject.wy = (newgameobject.y >> 6) + 2;
			if(newgameobject.wx < 0)
				newgameobject.wx = 0;
			if(newgameobject.wy >= MAP_LENGTH_Y - 1)
				newgameobject.wy = MAP_LENGTH_Y - 2;
			temp1 = newgameobject.wx;
			temp2 = newgameobject.wy;
			world[current_level][temp2][temp1].locallist.push_back(newgameobject);
		}
		// checks object collision only if not in running mode
		// also no wall detection checks, since behemoth can destroy walls
		if(gameobject.special1 >= 150) {
			move = Object_Collision_Object(gameobject.x0, gameobject.y0, gameobject.x1, gameobject.y1,
											gameobject.direction, gameobject.dx);
		}
		else
			move = 1;
		
		if(move == 1) {
			distance = Fast_Distance_2D((gameobject.x + 170 - player_x), (gameobject.y + 170 - player_y));
			gameobject.stuck = 0;
			gameobject.visible = true;
			if(distance <= 140) {
				gameobject.commit = 0;
				gameobject.special1 = 200;
				Set_Pos_BOB(&behemoth, gameobject.x - world_x, gameobject.y - world_y);
				behemoth.curr_frame = anims[gameobject.direction + 8][gameobject.curr_frame];
				Draw_BOB(&behemoth, lpddsback);
				if(gameobject.attack == 0) {
					DSound_Play(sounds[5]);
					if(gameobject.special1 > 150)
						player_health -= 20;
					else
						player_health -= 20;
					gameobject.attack = 30;
				}
				else
					gameobject.attack -= 1;
			}
			else {
				if(gameobject.direction >= NEAST && gameobject.direction <= SEAST && gameobject.wx < MAP_LENGTH_X -4) {
					gameobject.x += gameobject.dx;
					gameobject.x0 += gameobject.dx;
					gameobject.x1 += gameobject.dx;
				}
				else if(gameobject.direction >= SWEST && gameobject.direction <= NWEST && gameobject.wx > 2 ){
					gameobject.x -= gameobject.dx;
					gameobject.x0 -= gameobject.dx;
					gameobject.x1 -= gameobject.dx;
				}
				if(gameobject.direction >= NWEST && gameobject.direction <= NEAST && gameobject.wy > 5) {
					gameobject.y -= gameobject.dy;
					gameobject.y0 -= gameobject.dy;
					gameobject.y1 -= gameobject.dy;
				}
				else if((gameobject.direction == SWEST || gameobject.direction == SOUTH || gameobject.direction == SEAST) && gameobject.wy < MAP_LENGTH_Y -3){
					gameobject.y += gameobject.dy;
					gameobject.y0 += gameobject.dy;
					gameobject.y1 += gameobject.dy;
				}

				new_x = (gameobject.x0 >> 6);
				new_y = (gameobject.y1 >> 6);
				if((new_x != gameobject.wx) || (new_y != gameobject.wy)) {
					std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
					temp += index;
					temp1 = gameobject.wx;
					temp2 = gameobject.wy;
					gameobject.wx = new_x;
					gameobject.wy = new_y;
					world[current_level][new_y][new_x].locallist.push_back(gameobject);
					world[current_level][temp2][temp1].locallist.erase(temp);
					std::sort(world[current_level][new_y][new_x].locallist.begin(),
								world[current_level][new_y][new_x].locallist.end(), Sort_Objects);
				}
			}
		}
		else {
			gameobject.stuck += 1;
			gameobject.direction = rand()%8;
			gameobject.curr_frame = 0;
			gameobject.delay = 0;
			gameobject.commit = 30;
		}

		if(gameobject.visible && distance > 140) {
			Set_Pos_BOB(&behemoth, gameobject.x - world_x, gameobject.y - world_y);
			behemoth.curr_frame = anims[gameobject.direction][gameobject.curr_frame];
			Draw_BOB(&behemoth, lpddsback);
		}

		if(gameobject.stuck == 6)
			gameobject.visible = false;
	}
	else {
		if(gameobject.curr_frame != 3) {
			if(gameobject.delay < behemoth.anim_count_max + 5)
				gameobject.delay++;
			else {
				gameobject.delay = 0;
				gameobject.curr_frame++;
				if(gameobject.curr_frame == 3)
					gameobject.y += 230;
			}
		}
		if(violence == true) {
			Set_Pos_BOB(&behemoth, gameobject.x - world_x, gameobject.y - world_y);
			behemoth.curr_frame = anims[gameobject.direction][gameobject.curr_frame];
			Draw_BOB(&behemoth, lpddsback);
		}
	}
}
// creates new weapon object
void Weapon_Attack() {
	// gameobject, to create new weapon object, index x/y is the current map tile location for start of weapon
	GAME_OBJECT gameobject;
	int index_x = (player_x >> 6);
	int index_y = (player_y >> 6);
	// only create a weapon projectile object if delay is 0, prevent creation of projectile each frame
	if(player_weapon_delay == 0) {
		// create machine gun projectile
		if(player_weapon == MACHINE_GUN && machine_gun_ammo > 0) {
			if(DSound_Status_Sound(sounds[8]) != DSBSTATUS_PLAYING)
				DSound_Play(sounds[8]);
			// to enable display of weapon firing anaimation
			player_firing = true;
			machine_gun_ammo -= 1;
			// set new delay
			player_weapon_delay = 5;
			// init. projectile, load into map tile list
			gameobject.type = MACHINE_GUN;
			gameobject.x = player_x;
			gameobject.y = player_y - 10;
			gameobject.x0 = 0;
			gameobject.y0 = 0;
			gameobject.x1 = world_x + mouse_x + 31;
			gameobject.y1 = world_y + mouse_y + 33;
			gameobject.wx = index_x;
			gameobject.wy = index_y;
			gameobject.hit = false;
			gameobject.visible = false;
			gameobject.delay = 0;
			world[current_level][index_y][index_x].locallist.push_back(gameobject);
		}
		// similar to above, creates energy projectile
		else if(player_weapon == ENERGY_GUN && energy_gun_ammo > 0) {
			DSound_Play(sounds[4]);
			player_firing = true;
			energy_gun_ammo -= 1;
			player_weapon_delay = 15;
			gameobject.type = ENERGY_GUN;
			gameobject.x = player_x;
			gameobject.y = player_y - 10;
			gameobject.x0 = 0;
			gameobject.y0 = 0;
			gameobject.x1 = world_x + mouse_x + 31;
			gameobject.y1 = world_y + mouse_y + 33;
			gameobject.wx = index_x;
			gameobject.wy = index_y;
			gameobject.hit = false;
			gameobject.visible = false;
			gameobject.delay = 0;
			gameobject.newx = 0;
			gameobject.newy = 0;
			// special2 is distance from initial pos. to target location
			// special1 is incremental traveled distance so far, make projectile explode within a distance to
			// target if it did not hit anthing
			gameobject.special1 = 0;
			gameobject.special2 = 0;
			world[current_level][index_y][index_x].locallist.push_back(gameobject);
		}
		// create flame projectile, similar to above
		else if(player_weapon == FLAME_GUN && flame_gun_ammo > 0) {
			if(DSound_Status_Sound(sounds[1]) != DSBSTATUS_PLAYING)
				DSound_Play(sounds[1]);
			player_firing = true;
			flame_gun_ammo -= 1;
			player_weapon_delay = 4;
			gameobject.type = FLAME_GUN;
			gameobject.x = player_x;
			gameobject.y = player_y - 10;
			gameobject.x0 = 0;
			gameobject.y0 = 0;
			gameobject.x1 = world_x + mouse_x + 31;
			gameobject.y1 = world_y + mouse_y + 33;
			gameobject.wx = index_x;
			gameobject.wy = index_y;
			gameobject.hit = false;
			gameobject.visible = false;
			gameobject.delay = 0;
			gameobject.newx = 0;
			gameobject.newy = 0;
			gameobject.special1 = 0;
			world[current_level][index_y][index_x].locallist.push_back(gameobject);
		}
	}
}

void Machine_Gun_AI(GAME_OBJECT &gameobject, int index) {
	
	int startx, starty, endx, endy, temp1, temp2;

	if(gameobject.hit == true) {
		// draw bullet ricochet
		Set_Pos_BOB(&weapons, gameobject.x - world_x - 45, gameobject.y - world_y - 45);
		weapons.curr_frame = gameobject.curr_frame;
		Draw_BOB(&weapons, lpddsback);
		if(gameobject.delay >= weapons.anim_count_max - 2){
			gameobject.curr_frame += 1;
			gameobject.delay = 0;
		}
		else
			gameobject.delay += 1;
		// end of ricochet animation, delete object
		if(gameobject.curr_frame > 2) {
			std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
			temp += index;
			temp1 = gameobject.wx;
			temp2 = gameobject.wy;
			world[current_level][temp2][temp1].locallist.erase(temp);
		}
	}
	else {
		// for distance from initial pos. to dest. pos.
		int distance;
		// inital travel distance , distances in x and y directions
		float newx = 0,
			  newy = 0,
			  dx = gameobject.x1 - gameobject.x,
			  dy = gameobject.y1 - gameobject.y;
		
		// no distance, player target initial position
		if(dy == 0 && dx == 0)
			gameobject.hit = true;
		// trajectory is horizontal
		else if(dy == 0) {
			// finds absolute distance
			distance = abs(gameobject.x1 - gameobject.x);
			// no distance to travel in y
			dy = 0;
			// if end destination is to the left of player, else right
			if(gameobject.x1 < gameobject.x)
				dx = -1;
			else
				dx = 1;
		}
		// trajectory is vertical
		else if(dx == 0) {
			// finds absolute distance
			distance = abs(gameobject.y1 - gameobject.y);
			// no distance to travel in x
			dx = 0;
			// if end destination is above, else down
			if(gameobject.y1 < gameobject.y)
				dy = -1;
			else
				dy = 1;
		}
		// trjectory is diagonal
		else if(dx == dy) {	
			// finds absolute distance
			distance = abs(gameobject.x1 - gameobject.x);
			// x direction is to left, else right
			if(gameobject.x1 < gameobject.x)
				dx = -1;
			else
				dx = 1;
			// y direction is up, else down
			if(gameobject.y1 < gameobject.y)
				dy = -1;
			else
				dy = 1;
		}
		else {// not simple trajectory
			// if x distance is larger
			if(fabs(dx) > fabs(dy)) {
				// find abs x distance
				distance = abs(gameobject.x1 - gameobject.x);
				// if left, else right, temp holds x distance
				if(gameobject.x1 < gameobject.x) {
					temp1 = dx;
					dx = -1;
				}
				else{
					temp1 = dx;
					dx = 1;
				}
				// if y direction is up, else down, is a fractional increment since x distance is major
				if(gameobject.y1 < gameobject.y)
					dy = -fabs(dy/temp1);
				else
					dy = fabs(dy/temp1);
			}
			else {// same as above, except y is major distance
				distance = abs(gameobject.y1 - gameobject.y);
				if(gameobject.y1 < gameobject.y){
					temp1 = dy;
					dy = -1;
				}
				else{
					temp1 = dy;
					dy = 1;
				}
				if(gameobject.x1 < gameobject.x)
					dx = -fabs(dx/temp1);
				else
					dx = fabs(dx/temp1);
			}
		}
		// for initializing initial start of trajectory
		newx = gameobject.x;
		newy = gameobject.y;
		// checks hit detection every increment of checking trajectory line
		for(int d = 0; (d < distance) && (gameobject.hit == false); d++) {
			// initialized range of map tile lists to check around projectile after each increment
			// fixes array bounds
			startx = (gameobject.x >> 6) - 4;
			starty = (gameobject.y >> 6) - 4;

			endx = startx + 8;
			endy = starty + 8;
			if(startx < 0)
				startx = 0;
			else if(startx > MAP_LENGTH_X - 1)
				startx = MAP_LENGTH_X - 1;
			if(starty < 0)
				starty = 0;
			else if(starty > MAP_LENGTH_Y - 1)
				starty = MAP_LENGTH_Y - 1;

			if(endx < 0)
				endx = 0;
			else if(endx > MAP_LENGTH_X - 1)
				endx = MAP_LENGTH_X - 1;
			if(endy < 0)
				endy = 0;
			else if(endy > MAP_LENGTH_Y - 1)
				endy = MAP_LENGTH_Y - 1;

			// increments newx/newy by 1 for the major distance, and fractional for the
			// minor distance, similar to bresenhams line algorithm
			newx += dx;
			newy += dy;
			// update projectile location		
			gameobject.x = newx;
			gameobject.y = newy;
			//finds current map tile type
			temp1 = (gameobject.x >> 6);
			temp2 = (gameobject.y >> 6);
			// cycles through map tiles in area, and object lists for collisions
			// updates damage to hp, etc.
			for(int y = starty; y < endy; y++) {
				for(int x = startx; x < endx; x++) {
					for(int n = 0; n < world[current_level][y][x].locallist.size(); n++) {
						GAME_OBJECT &enemy = world[current_level][y][x].locallist[n];
						if(gameobject.x >= enemy.x0	&& gameobject.x <= enemy.x1 && enemy.visible == true) {
							if(gameobject.y >= (enemy.y0-30) && gameobject.y <= enemy.y1) {
								if(enemy.type == BLOB)
									enemy.hp += 4;
								else if(enemy.type == ZOMBIE)
									enemy.hp -= 15;
								else if(enemy.type == BEHEMOTH)
									enemy.hp -= 1;
								else if(enemy.type == BARREL_ITEM)
									enemy.hp -= 10;
								enemy.hit = true;
								gameobject.hit = true;
							}
						}
						// if current map tile is a wall, collision occured and end of object
						else if(world[current_level][temp2][temp1].type >= 0 && world[current_level][temp2][temp1].type <= 27)
							gameobject.hit = true;
					}
				}
			}
		}
		// if projectily has reached original distance to target, collision occurs
		if(DSound_Status_Sound(sounds[9]) != DSBSTATUS_PLAYING)
				DSound_Play(sounds[9]);
		gameobject.hit = true;
		gameobject.curr_frame = 0;
	}
}
// creates energy projectile, similar to machinegun project, comments specified for differances
void Energy_Gun_AI(GAME_OBJECT &gameobject, int index) {
	//GAME_OBJECT enemy;
	int startx, starty, endx, endy, temp1, temp2;

	if(gameobject.hit == true) {
		// draw bullet ricochet
		DSound_Play(sounds[7]);
		Set_Pos_BOB(&weapons, gameobject.x - world_x - 45, gameobject.y - world_y - 45);
		weapons.curr_frame = gameobject.curr_frame;
		Draw_BOB(&weapons, lpddsback);
		if(gameobject.delay >= weapons.anim_count_max - 3){
			gameobject.curr_frame += 1;
			gameobject.delay = 0;
		}
		else
			gameobject.delay += 1;
		if(gameobject.curr_frame > 11) {
			std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
			temp += index;
			temp1 = gameobject.wx;
			temp2 = gameobject.wy;
			world[current_level][temp2][temp1].locallist.erase(temp);
		}
	}
	else {// checks trajectory on initial processing of object, dx/dy distances, and updates accordingly
		if(gameobject.newx == 0 && gameobject.newy == 0) {
			int distance;
			float dx = gameobject.x1 - gameobject.x,
				  dy = gameobject.y1 - gameobject.y;
			
			if(dy == 0 && dx == 0)
				gameobject.hit = true;
			else if(dy == 0) {
				distance = abs(gameobject.x1 - gameobject.x);
				dy = 0;
				if(gameobject.x1 < gameobject.x)
					dx = -1;
				else
					dx = 1;
			}
			else if(dx == 0) {
				distance = abs(gameobject.y1 - gameobject.y);
				dx = 0;
				if(gameobject.y1 < gameobject.y)
					dy = -1;
				else
					dy = 1;
			}
			else if(dx == dy) {	
				distance = abs(gameobject.x1 - gameobject.x);
				if(gameobject.x1 < gameobject.x)
					dx = -1;
				else
					dx = 1;
				if(gameobject.y1 < gameobject.y)
					dy = -1;
				else
					dy = 1;
			}
			else {
				if(fabs(dx) > fabs(dy)) {
					distance = abs(gameobject.x1 - gameobject.x);
					if(gameobject.x1 < gameobject.x) {
						temp1 = dx;
						dx = -1;
					}
					else{
						temp1 = dx;
						dx = 1;
					}
					if(gameobject.y1 < gameobject.y)
						dy = -fabs(dy/temp1);
					else
						dy = fabs(dy/temp1);
				}
				else {
					distance = abs(gameobject.y1 - gameobject.y);
					if(gameobject.y1 < gameobject.y){
						temp1 = dy;
						dy = -1;
					}
					else{
						temp1 = dy;
						dy = 1;
					}
					if(gameobject.x1 < gameobject.x)
						dx = -fabs(dx/temp1);
					else
						dx = fabs(dx/temp1);
				}
			}
			// projectile does not move pixel by pixel, but on scale of 8 pixels
			dx *= 8;
			dy *= 8;
			// keeps initial distance to target
			gameobject.special2 = distance;
			gameobject.newx = gameobject.x;
			gameobject.newy = gameobject.y;
			gameobject.edx = dx;
			gameobject.edy = dy;
		}
		// creates rect range of map tiles centered around player
		startx = (gameobject.x >> 6) - 4;
		starty = (gameobject.y >> 6) - 4;

		endx = startx + 8;
		endy = starty + 8;
		if(startx < 0)
			startx = 0;
		else if(startx > MAP_LENGTH_X - 1)
			startx = MAP_LENGTH_X - 1;
		if(starty < 0)
			starty = 0;
		else if(starty > MAP_LENGTH_Y - 1)
			starty = MAP_LENGTH_Y - 1;

		if(endx < 0)
			endx = 0;
		else if(endx > MAP_LENGTH_X - 1)
			endx = MAP_LENGTH_X - 1;
		if(endy < 0)
			endy = 0;
		else if(endy > MAP_LENGTH_Y - 1)
			endy = MAP_LENGTH_Y - 1;
		
		// updates change in x and y, edx edy are intial calculate veloity found on first run through AI
		gameobject.newx += gameobject.edx;
		gameobject.newy += gameobject.edy;
							
		gameobject.x = gameobject.newx;
		gameobject.y = gameobject.newy;
		temp1 = (gameobject.x >> 6);
		temp2 = (gameobject.y >> 6);
		// updates travel so far
		gameobject.special1 += 8;
		if(gameobject.special1 >= 50) {
			Set_Pos_BOB(&weapons, gameobject.x - world_x - 45, gameobject.y - world_y - 45);
			weapons.curr_frame = 3;
			Draw_BOB(&weapons, lpddsback);
		}

		for(int y = starty; y < endy; y++) {
			for(int x = startx; x < endx; x++) {
				for(int n = 0; n < world[current_level][y][x].locallist.size(); n++) {
					GAME_OBJECT &enemy = world[current_level][y][x].locallist[n];
					if(gameobject.x >= enemy.x0	&& gameobject.x <= enemy.x1 && enemy.visible == true) {
						if(gameobject.y >= (enemy.y0-30) && gameobject.y <= enemy.y1) {
							if(enemy.type == BLOB)
								enemy.hp += 20;
							else if(enemy.type == ZOMBIE)
								enemy.hp -= 100;
							else if(enemy.type == BEHEMOTH)
								enemy.hp -= 25;
							else
								enemy.hp -= 30;
							enemy.hit = true;
							gameobject.hit = true;
							gameobject.curr_frame = 4;
						}
					}
					else if(world[current_level][temp2][temp1].type >= 0 && world[current_level][temp2][temp1].type <= 27) {
						gameobject.hit = true;
						gameobject.curr_frame = 4;
					}
				}
			}
		}
		if(gameobject.special1 >= gameobject.special2) {
			gameobject.hit = true;
			gameobject.curr_frame = 4;
		}
	}
}
// AI for each flame projectile for flame thrower weapon, similar to above machine gun AI
void Flame_Gun_AI(GAME_OBJECT &gameobject, int index) {
	//GAME_OBJECT enemy;
	int startx, starty, endx, endy, temp1, temp2;

	if(gameobject.hit == true) {
		// draw bullet ricochet
		Set_Pos_BOB(&weapons, gameobject.x - world_x - gameobject.x0, gameobject.y - world_y - gameobject.x0);
		weapons.curr_frame = 17;
		Draw_Scaled_BOB(&weapons, 2 * gameobject.x0, 2 * gameobject.x0, lpddsback);
		if(gameobject.x0 >= 3)
			gameobject.x0 -= 4;
		else {
			std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
			temp += index;
			temp1 = gameobject.wx;
			temp2 = gameobject.wy;
			world[current_level][temp2][temp1].locallist.erase(temp);
		}
	}
	else {
		if(gameobject.newx == 0 && gameobject.newy == 0) {
			int distance;
			distance = Slope_Direction(gameobject.x, gameobject.y, gameobject.x1, gameobject.y1);
			if(distance == 0 || distance == 2 || distance == 4 || distance == 6)
				gameobject.special2 = 250;
			else
				gameobject.special2 = 200;

			float dx = gameobject.x1 - gameobject.x,
				  dy = gameobject.y1 - gameobject.y;
			
			if(dy == 0 && dx == 0)
				gameobject.hit = true;
			else if(dy == 0) {
				distance = abs(gameobject.x1 - gameobject.x);
				dy = 0;
				if(gameobject.x1 < gameobject.x)
					dx = -1;
				else
					dx = 1;
			}
			else if(dx == 0) {
				distance = abs(gameobject.y1 - gameobject.y);
				dx = 0;
				if(gameobject.y1 < gameobject.y)
					dy = -1;
				else
					dy = 1;
			}
			else if(dx == dy) {	
				distance = abs(gameobject.x1 - gameobject.x);
				if(gameobject.x1 < gameobject.x)
					dx = -1;
				else
					dx = 1;
				if(gameobject.y1 < gameobject.y)
					dy = -1;
				else
					dy = 1;
			}
			else {
				if(fabs(dx) > fabs(dy)) {
					distance = abs(gameobject.x1 - gameobject.x);
					if(gameobject.x1 < gameobject.x) {
						temp1 = dx;
						dx = -1;
					}
					else{
						temp1 = dx;
						dx = 1;
					}
					if(gameobject.y1 < gameobject.y)
						dy = -fabs(dy/temp1);
					else
						dy = fabs(dy/temp1);
				}
				else {
					distance = abs(gameobject.y1 - gameobject.y);
					if(gameobject.y1 < gameobject.y){
						temp1 = dy;
						dy = -1;
					}
					else{
						temp1 = dy;
						dy = 1;
					}
					if(gameobject.x1 < gameobject.x)
						dx = -fabs(dx/temp1);
					else
						dx = fabs(dx/temp1);
				}
			}
			// travels 6 pixels on major x or y distance, instead of 1 at a time
			dx *= 6;
			dy *= 6;
			gameobject.newx = gameobject.x;
			gameobject.newy = gameobject.y;
			gameobject.edx = dx;
			gameobject.edy = dy;
			gameobject.x0 = 0;
		}
		
		startx = (gameobject.x >> 6) - 4;
		starty = (gameobject.y >> 6) - 4;

		endx = startx + 8;
		endy = starty + 8;
		if(startx < 0)
			startx = 0;
		else if(startx > MAP_LENGTH_X - 1)
			startx = MAP_LENGTH_X - 1;
		if(starty < 0)
			starty = 0;
		else if(starty > MAP_LENGTH_Y - 1)
			starty = MAP_LENGTH_Y - 1;

		if(endx < 0)
			endx = 0;
		else if(endx > MAP_LENGTH_X - 1)
			endx = MAP_LENGTH_X - 1;
		if(endy < 0)
			endy = 0;
		else if(endy > MAP_LENGTH_Y - 1)
			endy = MAP_LENGTH_Y - 1;

		gameobject.newx += gameobject.edx;
		gameobject.newy += gameobject.edy;
							
		gameobject.x = gameobject.newx;
		gameobject.y = gameobject.newy;
		temp1 = (gameobject.x >> 6);
		temp2 = (gameobject.y >> 6);
		// update distance traveled so far
		gameobject.special1 += 6;
		if(gameobject.special1 >= 40 || gameobject.hit == true) {
			Set_Pos_BOB(&weapons, gameobject.x - world_x - gameobject.x0, gameobject.y - world_y - gameobject.x0);
			weapons.curr_frame = 17;
			gameobject.x0+= 2;
			Draw_Scaled_BOB(&weapons,2 * gameobject.x0, 2* gameobject.x0, lpddsback);
		}

		for(int y = starty; y < endy; y++) {
			for(int x = startx; x < endx; x++) {
				for(int n = 0; n < world[current_level][y][x].locallist.size(); n++) {
					GAME_OBJECT &enemy = world[current_level][y][x].locallist[n];
					if(gameobject.x >= enemy.x0	&& gameobject.x <= enemy.x1 && enemy.visible == true) {
						if(gameobject.y >= (enemy.y0-30) && gameobject.y <= enemy.y1) {
							if(enemy.type == ZOMBIE)
								enemy.hp -= 34;
							else if(enemy.type == BLOB)
								enemy.hp -= 10;
							else if(enemy.type == SPIDER_BLOB)
								enemy.hp -= 20;
							else if(enemy.type == BARREL_ITEM)
								enemy.hp -= 10;
							enemy.hit = true;
							gameobject.curr_frame = 4;
							gameobject.hit = true;
						}
					}
					else if(world[current_level][temp2][temp1].type >= 0 && world[current_level][temp2][temp1].type <= 27) {
						gameobject.hit = true;
						gameobject.curr_frame = 4;
					}
				}
			}
		}
		if(gameobject.special1 >= gameobject.special2) {
			gameobject.hit = true;
			gameobject.curr_frame = 4;
		}
	}
}
// processes AI for each game object icon... ammo/hp/key item
void Item_AI(GAME_OBJECT &gameobject, int index) {
	// if taken becomes true, delete object
	bool taken = false;
	int temp1, temp2;
	// checks which item to evaluate
	switch(gameobject.type) {
		// if player walks over each of these icons, increments health or ammo if not at max, 
		// fixes if over max, make taken true
		case HP_ITEM : {
			if(player_x >= gameobject.x0 && player_x <= gameobject.x1 
			&& player_y >= gameobject.y0 && player_y <= gameobject.y1)
				if(player_health < 100) {
					player_health += 20;
					taken = true;
					DSound_Play(sounds[14]);
					if(player_health > 100)
						player_health = 100;
			}
			break;
					   }
			
		case MACHINEAMMO_ITEM : {
			if(player_x >= gameobject.x0 && player_x <= gameobject.x1 
			&& player_y >= gameobject.y0 && player_y <= gameobject.y1)
				if(machine_gun_ammo < 200) {
					machine_gun_ammo += 25;
					taken = true;
					DSound_Play(sounds[14]);
					if(machine_gun_ammo > 200)
						machine_gun_ammo = 200;
			}
			break;
					   }

		case ENERGYAMMO_ITEM : {
			if(player_x >= gameobject.x0 && player_x <= gameobject.x1 
			&& player_y >= gameobject.y0 && player_y <= gameobject.y1)
				if(energy_gun_ammo < 30) {
					energy_gun_ammo += 10;
					taken = true;
					DSound_Play(sounds[14]);
					if(energy_gun_ammo > 30)
						energy_gun_ammo = 30;
			}
			break;
					   }
		case FLAMEAMMO_ITEM : {
			if(player_x >= gameobject.x0 && player_x <= gameobject.x1 
			&& player_y >= gameobject.y0 && player_y <= gameobject.y1)
				if(flame_gun_ammo < 100) {
					flame_gun_ammo += 20;
					taken = true;
					DSound_Play(sounds[14]);
					if(flame_gun_ammo > 100)
						flame_gun_ammo = 100;
			}
			break;
					   }
		// key items to find, increments total number found so far
		case KEY_ITEM : {
			if(player_x >= gameobject.x0 && player_x <= gameobject.x1 
				&& player_y >= gameobject.y0 && player_y <= gameobject.y1) {
				DSound_Play(sounds[14]);
				key_item++;
				taken = true;
			}
			break;
					   }
		// if hp < 0, creates and inits. the barrel explosion object
		case BARREL_ITEM : {
			if(gameobject.hp <= 0) {
				if(gameobject.special1 <= 0) {
					taken = true;
					DSound_Play(sounds[2]);
					GAME_OBJECT newgameobject;
					newgameobject.type = LARGE_EXPLOSION;
					newgameobject.visible = false;
					newgameobject.hit = false;
					newgameobject.x = gameobject.x - 128;
					newgameobject.y = gameobject.y - 128;
					newgameobject.y1 = 0;
					newgameobject.delay = 0;
					newgameobject.curr_frame = 0;
					newgameobject.wx = gameobject.wx - 2;
					newgameobject.wy = gameobject.wy + 2;
					if(newgameobject.wx < 0)
						newgameobject.wx = 0;
					if(newgameobject.wy >= MAP_LENGTH_Y - 1)
						newgameobject.wy = MAP_LENGTH_Y - 2;
					temp1 = newgameobject.wx;
					temp2 = newgameobject.wy;
					world[current_level][temp2][temp1].locallist.push_back(newgameobject);
				}
				else
					gameobject.special1--;
			}
			break;
					   }

		default: break;
	}
	// if taken, destroyed if barrel, deletes object
	if(taken) {
		std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
		temp += index;
		temp1 = gameobject.wx;
		temp2 = gameobject.wy;
		world[current_level][temp2][temp1].locallist.erase(temp);
	}
	// draws object
	else if(gameobject.curr_frame!=3) {
		Set_Pos_BOB(&weapons, gameobject.x - world_x, gameobject.y - world_y);
		weapons.curr_frame = gameobject.curr_frame;
		Draw_BOB(&weapons, lpddsback);
	}
}
// processes an explosions AI
void Explosion_AI(GAME_OBJECT &gameobject, int index) {
	// if its a barrel explosion and this is the first pass of its AI
	if(gameobject.hit == false && gameobject.type == LARGE_EXPLOSION) {
		// sets true so this part is not processed again
		gameobject.hit = true;
		// creates a rect range of map tiles to check, centered around middle of explosion
		int startx, starty, endx, endy;
		startx = (gameobject.x >> 6) - 3;
		starty = (gameobject.y >> 6) - 3;

		endx = startx + 8;
		endy = starty + 8;
		if(startx < 0)
			startx = 0;
		else if(startx > MAP_LENGTH_X - 1)
			startx = MAP_LENGTH_X - 1;
		if(starty < 0)
			starty = 0;
		else if(starty > MAP_LENGTH_Y - 1)
			starty = MAP_LENGTH_Y - 1;

		if(endx < 0)
			endx = 0;
		else if(endx > MAP_LENGTH_X - 1)
			endx = MAP_LENGTH_X - 1;
		if(endy < 0)
			endy = 0;
		else if(endy > MAP_LENGTH_Y - 1)
			endy = MAP_LENGTH_Y - 1;
		// checks objects in all map tile list within range, hurts hp of each kind, (enemy or another barrel)
		for(int y = starty; y < endy; y++) {
			for(int x = startx; x < endx; x++) {
				for(int n = 0; n < world[current_level][y][x].locallist.size(); n++) {
					GAME_OBJECT &enemy = world[current_level][y][x].locallist[n];
					if(Rect_Collision(enemy.x0, enemy.y0, enemy.x1, enemy.y1,
									gameobject.x, gameobject.y, gameobject.x + 400, gameobject.y + 400))
					{
						if(gameobject.type == BEHEMOTH)
							enemy.hp -= 30;
						else
							enemy.hp -= 100;
						enemy.hit = true;
					}
				}
			}
		}
		// checks if player is in range of explosion
		if((player_x >= gameobject.x && player_x <= (gameobject.x + 360)) &&
			(player_y >= gameobject.y && player_y <= (gameobject.y +360))) {
			DSound_Play(sounds[5]);
			player_health -= 75;
		}
	}
	// large explosion, barrel explosion
	if(gameobject.type == LARGE_EXPLOSION) {
		// if in last frame, deletes object
		if(gameobject.curr_frame == 11) {
			int temp1, temp2;
			std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
			temp += index;
			temp1 = gameobject.wx;
			temp2 = gameobject.wy;
			world[current_level][temp2][temp1].locallist.erase(temp);
		}
		else {
			// increments through frames, draws explosion
			if(gameobject.delay < 4)
				gameobject.delay++;
			else {
				gameobject.delay = 0;
				gameobject.curr_frame++;
			}
			if(gameobject.curr_frame < 11) {
				Set_Pos_BOB(&explosions, gameobject.x - world_x, gameobject.y - world_y);
				explosions.curr_frame = gameobject.curr_frame;
				Draw_BOB(&explosions, lpddsback);
			}
		}
	}
	// wall explosion
	if(gameobject.type == SMALL_EXPLOSION) {
		// if in last frame, deletes object
		if(gameobject.curr_frame == 17) {
			int temp1, temp2;
			std::vector<GAME_OBJECT>::iterator temp = world[current_level][gameobject.wy][gameobject.wx].locallist.begin();
			temp += index;
			temp1 = gameobject.wx;
			temp2 = gameobject.wy;
			world[current_level][temp2][temp1].locallist.erase(temp);
		}
		else {
			// increments through frames, draws explosion
			if(gameobject.delay < 4)
				gameobject.delay++;
			else {
				gameobject.delay = 0;
				gameobject.curr_frame++;
			}
			if(gameobject.curr_frame < 17) {
				Set_Pos_BOB(&explosions, gameobject.x - world_x, gameobject.y - world_y);
				explosions.curr_frame = gameobject.curr_frame;
				Draw_BOB(&explosions, lpddsback);
			}
		}
	}
}
// checks if values are outside the bounds of the world array, or one level
void Fix_Bounds(int &x, int &y) {
	if(x < 0)
		x = 0;
	else if(x > MAP_LENGTH_X - 1)
		x = MAP_LENGTH_X - 1;

	if(y < 0)
		y = 0;
	else if(y > MAP_LENGTH_Y - 1)
		y = MAP_LENGTH_Y - 1;
}
